from __future__ import annotations

from tcsoa.gen.Vendormanagement._2008_06.VendorManagement import ChangeVendorInputInfo, GetVendorPartsWithSelRuleResponse, GetVendorPartsWithSelRuleInputInfo, ChangeVendorResponse
from typing import List
from tcsoa.base import TcService


class VendorManagementService(TcService):

    @classmethod
    def changeVendor(cls, input: List[ChangeVendorInputInfo]) -> ChangeVendorResponse:
        """
        This service operation can be called on either a VendorPart or a Vendor.
        
        For VendorPart: It creates a new copy of VendorPart with the new Vendor information. This newly created
        VendorPart is associated to the new revision of the CommercialPart of the old VendorPart.
        For Vendor: All VendorPart objects related with given Vendor are taken into consideration. Each of these
        VendorPart objects are then processed as mentioned above.
        
        This operation shows old and new part id with the information of the success or failure. 
        
        Typical Client Usage: 
        Typical usage involves two Vendor objects and a VendorPart created with either one of them.
        
        'VendorManagementService vmService = 
        VendorManagementService.getService(session);
        ChangeVendorInputInfo[]chanProps = new ChangeVendorInputInfo[1];
        ChangeVendorInputInfo changeProps = new ChangeVendorInputInfo();
        ChangeVendorResponse response = null;
        changeProps.newVendor = (TCComponentVendor)newVendor;
        changeProps.vendorParts=selectedParts;
        chanProps[0] = changeProps;
        
        response = vmService.changeVendor( chanProps );'
        """
        return cls.execute_soa_method(
            method_name='changeVendor',
            library='Vendormanagement',
            service_date='2008_06',
            service_name='VendorManagement',
            params={'input': input},
            response_cls=ChangeVendorResponse,
        )

    @classmethod
    def getVendorPartsWithSelRule(cls, input: List[GetVendorPartsWithSelRuleInputInfo]) -> GetVendorPartsWithSelRuleResponse:
        """
        This service operation returns VendorPart objects  associated with the given CommercialPartRevision  based on
        the selection rule set for the VendorPart objects in Structure Manager. If no value is given for the selection
        rule, it is read  from the preference VMS_vendor_part_selection_rule. Typical OOTB selection rules provided are
        showAllVendorParts and showPreferredVendorPartsOnly.  First selection rule is to show all related VendorPart
        objects and later is used to show the related VendorPart objects carrying preferred status.
        """
        return cls.execute_soa_method(
            method_name='getVendorPartsWithSelRule',
            library='Vendormanagement',
            service_date='2008_06',
            service_name='VendorManagement',
            params={'input': input},
            response_cls=GetVendorPartsWithSelRuleResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessModeler._2010_04.BusinessRules import GetVerificationRulesResponse, VerificationRuleInput
from typing import List
from tcsoa.base import TcService


class BusinessRulesService(TcService):

    @classmethod
    def getVerificationRules(cls, input: List[VerificationRuleInput]) -> GetVerificationRulesResponse:
        """
        This operation gets a list of VerificationRule objects which match the given criteria each
        VerificationRuleInput supplied and its context_filter attribute which refers to a Condition object defined in
        Business Modeler IDE equals to true; Wild cards can be specified in strings using * in VerificationRuleInput
        which mean to match all cases for the criteria strings. This operation not only returns all VerificationRule
        objects which full match criteria in VerificationRuleInput, it also returns VerificationRule objects whose type
        attribute is the parent of the typeName defined in the VerificationRuleInput, meanwhile other criteria are
        matched.
        
        Use cases:
        4 samples Verification Rules created in Business Modeler IDE
        {Functionality= Fnd0BOMGrading, Business Object=Item, Context Filter =isTrue, Condition Rule=Condition1, Sub
        Group=*}
        {Functionality= Fnd0BOMGrading, Business Object=Part, Context Filter = isEngineeringGroup, Condition
        Rule=Condition1, Sub Group=*}
        {Functionality= Fnd0BOMGrading, Business Object= CommericalPart, Context Filter =isTrue, Condition
        Rule=Condition2, Sub Group= Americas}
        {Functionality= Fnd0APS, Business Object=Item, Context Filter =isTrue, Condition Rule=Condition1, Sub Group= *}
        Login Teamcenter as dba group and get VerificationRule objects by setting functionality as Fnd0BOMGrading and
        typeName as CommericalPart. Following VerificationRule objects in will be returned.
        {Functionality= Fnd0BOMGrading, Business Object=Item, Context Filter =isTrue, Condition Rule=Condition1, Sub
        Group=*}
        {Functionality= Fnd0BOMGrading, Business Object= CommericalPart, Context Filter =isTrue, Condition
        Rule=Condition2, Sub Group=Americas}
        The first rule applies to Item, if Condition1 could check Item, it could check CommericalPart, since
        CommericalPart is a sub type of Item and it is sure that CommericalPart contains all properties checked in
        Condition1, so it is valid. The second rule applies to all CommericalPart, so it is also valid. The other
        VerificationRule objects either do not match functionality or typeName, or else context_filter equals to false.
        """
        return cls.execute_soa_method(
            method_name='getVerificationRules',
            library='BusinessModeler',
            service_date='2010_04',
            service_name='BusinessRules',
            params={'input': input},
            response_cls=GetVerificationRulesResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Manufacturing._2011_06.DataManagement import OpenViewsResponse, OpenContextInput, OpenContextsResponse
from tcsoa.gen.Manufacturing._2011_06.StructureManagement import ReferencedContextsResponse, ReferencedContexts
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class StructureManagementService(TcService):

    @classmethod
    def getReferenceContexts(cls, contexts: List[BusinessObject]) -> ReferencedContextsResponse:
        """
        return referenced contexts of input context
        """
        return cls.execute_soa_method(
            method_name='getReferenceContexts',
            library='Manufacturing',
            service_date='2011_06',
            service_name='StructureManagement',
            params={'contexts': contexts},
            response_cls=ReferencedContextsResponse,
        )

    @classmethod
    def setReferenceContexts(cls, input: List[ReferencedContexts]) -> ServiceData:
        """
        set Reference context according to user choice
        """
        return cls.execute_soa_method(
            method_name='setReferenceContexts',
            library='Manufacturing',
            service_date='2011_06',
            service_name='StructureManagement',
            params={'input': input},
            response_cls=ServiceData,
        )


class DataManagementService(TcService):

    @classmethod
    def closeContexts(cls, contexts: List[BusinessObject]) -> ServiceData:
        """
        This method is used to close contexts (base view windows). For each context, this method closes the base view
        window and all the open views (OG windows) associated to it
        """
        return cls.execute_soa_method(
            method_name='closeContexts',
            library='Manufacturing',
            service_date='2011_06',
            service_name='DataManagement',
            params={'contexts': contexts},
            response_cls=ServiceData,
        )

    @classmethod
    def closeViews(cls, structureContext: BusinessObject, views: List[BusinessObject]) -> ServiceData:
        """
        This method is used to close opened views (OG windows)
        """
        return cls.execute_soa_method(
            method_name='closeViews',
            library='Manufacturing',
            service_date='2011_06',
            service_name='DataManagement',
            params={'structureContext': structureContext, 'views': views},
            response_cls=ServiceData,
        )

    @classmethod
    def openContexts(cls, input: List[OpenContextInput]) -> OpenContextsResponse:
        """
        This method is used to open existing objects in new base view windows
        """
        return cls.execute_soa_method(
            method_name='openContexts',
            library='Manufacturing',
            service_date='2011_06',
            service_name='DataManagement',
            params={'input': input},
            response_cls=OpenContextsResponse,
        )

    @classmethod
    def openViews(cls, context: BusinessObject, structureContext: BusinessObject, views: List[BusinessObject]) -> OpenViewsResponse:
        """
        This method is used to open views (OG windows) for an already opened context (base view window).
        """
        return cls.execute_soa_method(
            method_name='openViews',
            library='Manufacturing',
            service_date='2011_06',
            service_name='DataManagement',
            params={'context': context, 'structureContext': structureContext, 'views': views},
            response_cls=OpenViewsResponse,
        )



from __future__ import annotations

from tcsoa.gen.Internal.StructureManagement._2017_05.StructureVerification import AttachmentComparisonDetailsResponse
from tcsoa.gen.Internal.StructureManagement._2017_05.StructureExpansionLite import Controls2, Controls3, ExpansionResponse3
from tcsoa.gen.Internal.StructureManagement._2017_05.StructureLiteConversion import ConversionResponse
from tcsoa.gen.StructureManagement._2012_02.StructureVerification import EquivalentLines
from typing import List
from tcsoa.gen.BusinessObjects import RuntimeBusinessObject
from tcsoa.gen.StructureManagement._2012_10.StructureVerification import StringToPartialMatchCriteria
from tcsoa.base import TcService


class StructureExpansionLiteService(TcService):

    @classmethod
    def getUndelivered3(cls, undeliveredLineUids: List[str], controls: Controls3) -> ExpansionResponse3:
        """
        This operation returns Fnd0BomLineLite as objects for input UIDs of Fnd0BomLineLite. 
        It may be required to invoke this operation more than once as the number of Fnd0BomLineLite returned depends on
        outputPageSize specified in 'Controls'. The 'RelationAndTypesCriteria' of 'Controls' is used to retrieve
        Dataset, named references objects with FMS tickets when named reference object is file.
        
        Use cases:
        This operation helps while working with large structure. 
        
        1. Client creates BOMWindow with RevisionRule and sets top BOMLine.
        2. Client optionally sets Variant Rule on BOMWindow.
        3. Client invokes expandNext3 operation with top BOMLine to get next set of configured children as
        Fnd0BomLineLite objects.
        4. Client checks if there are any undelivered Fnd0BomLineLite objects in ExpansionResponse.
        5. Client invokes getUndelivered3 operation to get next set of undelivered Fnd0BomLineLites along with Dataset.
        6. Client uses response from expandNext3 operation to create a configured structure including Dataset
        information on client and Dataset to render the product.
        7. Client closes the BOMWindow to release resources.
        """
        return cls.execute_soa_method(
            method_name='getUndelivered3',
            library='Internal-StructureManagement',
            service_date='2017_05',
            service_name='StructureExpansionLite',
            params={'undeliveredLineUids': undeliveredLineUids, 'controls': controls},
            response_cls=ExpansionResponse3,
        )

    @classmethod
    def expandNext3(cls, linesToExpand: List[RuntimeBusinessObject], controls: Controls2) -> ExpansionResponse3:
        """
        The operation performs top down expansion for the given parent Fnd0BOMLineLite or BOMLine. The level of
        expansion is based upon input 'Controls' such as maxLevel. The 'RelationAndTypesCriteria' of 'Controls' is used
        to retrieve Dataset, named references objects with FMS tickets when named reference object is file.
        Note that partial expansion for ImanItemBOPLine is not supported.
        
        Use cases:
        This operation is suited to get configured structure and related Dataset for read-only scenarios, e.g.
        Visualization, NX. 
        
        1. Client creates BOMWindow with RevisionRule and sets top BOMLine.
        2. Client optionally sets Variant Rule on BOMWindow.
        3. Client invokes expandNext3 operation with top BOMLine to get next set of configured children as
        Fnd0BOMLineLite objects.
        4. Client invokes expandNext3 recursively until till the structure is fully completely expanded.
        5. Client uses response from expandNext3 operation to create a configured structure including Dataset
        information on client and Dataset to render the product.
        6. Client closes the BOMWindow to release resources.
        """
        return cls.execute_soa_method(
            method_name='expandNext3',
            library='Internal-StructureManagement',
            service_date='2017_05',
            service_name='StructureExpansionLite',
            params={'linesToExpand': linesToExpand, 'controls': controls},
            response_cls=ExpansionResponse3,
        )


class StructureLiteConversionService(TcService):

    @classmethod
    def liteBOMLinesToBOMLines(cls, linesToConvert: List[RuntimeBusinessObject]) -> ConversionResponse:
        """
        The operation converts the given list of Fnd0BOMLineLite to list of BOMLine. It also converts all the
        Fnd0BOMLineLite parents of the input Fnd0BOMLineLite till until it finds the BOMLine parent in the same
        hierarchy. No Fnd0BOMLineLite objects from same parent hierarchy are expected.
        
        Use cases:
        1. Client creates BOMWindow with RevisionRule and sets top BOMLine.
        2. Client optionally sets Variant Rule on BOMWindow.
        3. Client invokes 'expandNext3' with top BOMLine to get next set of configured children as Fnd0BOMLineLite
        objects.
        4. Client optionally invokes 'expandNext3' recursively till structure is fully expanded.
        5. Client uses 'liteBOMLinesToBOMLines' operation to convert a Fnd0BOMLineLite line to BOMLine. This line can
        be from any level of assembly but single Fnd0BOMLineLite line should be passed from the same parent hierarchy.
        6. Client closes the BOMWindow to release resources.
        """
        return cls.execute_soa_method(
            method_name='liteBOMLinesToBOMLines',
            library='Internal-StructureManagement',
            service_date='2017_05',
            service_name='StructureLiteConversion',
            params={'linesToConvert': linesToConvert},
            response_cls=ConversionResponse,
        )


class StructureVerificationService(TcService):

    @classmethod
    def getAttachmentComparisonDetails(cls, equivalentObjects: List[EquivalentLines], partialMatchCriteria: List[StringToPartialMatchCriteria]) -> AttachmentComparisonDetailsResponse:
        """
        This operation returns the details of any differences between attachments for the supplied source and target
        BOMLine objects. Attachments can be Dataset or Form objects. The operation takes the source and target BOMLine
        objects and compares their attachments according to their types &ndash; Dataset are compared with Dataset and
        Form with Form. The supported attachment relation types to compare are governed by
        MEAccountabilityCheckIncludedAttachmentRelations site preference such as the relation type Cps0LBRel for the
        dataset ProcessSimulatePLCLB etc. The source and target attachments are returned by this operation in the form
        of a table that is created by the output structures.
        
        Use cases:
        A user wants to perform comparison of Attachments within a particular scope.
        """
        return cls.execute_soa_method(
            method_name='getAttachmentComparisonDetails',
            library='Internal-StructureManagement',
            service_date='2017_05',
            service_name='StructureVerification',
            params={'equivalentObjects': equivalentObjects, 'partialMatchCriteria': partialMatchCriteria},
            response_cls=AttachmentComparisonDetailsResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import EPMJob, Fnd0ProxyTask, WorkspaceObject, POM_object, ScheduleTask, ResourceAssignment, TaskDependency, User, Discipline
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class AssignmentCreateContainer(TcBaseObj):
    """
    The information needed to create a new assignment to a task.
    
    :var task: The task to assign the resource.
    :var user: The user being assigned (or null if discipline assignment).
    :var discipline: The discipline of the assignee or the discipline for the assignment.
    :var assignedPercent: The percentage effort being assigned (I.E. 50.0 == 50%)
    """
    task: ScheduleTask = None
    user: User = None
    discipline: Discipline = None
    assignedPercent: float = 0.0


@dataclass
class AssignmentUpdateContainer(TcBaseObj):
    """
    Container for assignment updates.
    
    :var assignment: The assignment to update
    :var newEffort: The new % effort of that assignee.
    """
    assignment: ResourceAssignment = None
    newEffort: float = 0.0


@dataclass
class LaunchedWorkflowContainer(TcBaseObj):
    """
    Information about the launched workflows.
    
    :var launchedWorkflows: The launched workflow processes.
    :var serviceData: The service data.
    """
    launchedWorkflows: List[EPMJob] = ()
    serviceData: ServiceData = None


@dataclass
class MoveRequest(TcBaseObj):
    """
    The information required to move a task.
    
    :var task: The task or proxy task to move.
    :var newParent: The new parent for the task.
    :var prevSibling: The new previous sibling for the task (or the reference task in the case of a proxy).
    """
    task: WorkspaceObject = None
    newParent: ScheduleTask = None
    prevSibling: ScheduleTask = None


@dataclass
class ObjectUpdateContainer(TcBaseObj):
    """
    Information to update an object.
    
    :var object: The object being updated.
    :var updates: The updates to the main type.
    :var typedUpdates: Updates for a custom type.
    """
    object: POM_object = None
    updates: List[AttributeUpdateContainer] = ()
    typedUpdates: List[TypedAttributeUpdateContainer] = ()


@dataclass
class AttributeUpdateContainer(TcBaseObj):
    """
    Holds the name and value of an attribute.
    
    :var attrName: The name of the attribute.
    :var attrValue: The value of the attribute.
    :var attrType: An integer to help determine the data type of the attribute. Valid values- {
    1=String_type,2=Integer_type,3=Long_type,4=Double_type,5=Float_type,6=Boolean_type,7=Date_type,8=Cal_type }
    """
    attrName: str = ''
    attrValue: str = ''
    attrType: int = 0


@dataclass
class TaskCreateContainer(TcBaseObj):
    """
    The information needed to create a task.
    
    :var name: The name.
    :var desc: The description.
    :var objectType: The type to create (typically "ScheduleTask").
    :var start: The start date and time.
    :var finish: The finish date and time.
    :var workEstimate: The work estimate in minutes.
    :var parent: The summary task to create the new task under.
    :var prevSibling: The task to create this new task after.  Must currently exist under the parent.  Can be null.
    :var otherAttributes: Additional optional attributes: priority
    :var typedOtherAttributes: Other attributes required to create the objectType specified.
    """
    name: str = ''
    desc: str = ''
    objectType: str = ''
    start: datetime = None
    finish: datetime = None
    workEstimate: int = 0
    parent: ScheduleTask = None
    prevSibling: ScheduleTask = None
    otherAttributes: List[AttributeUpdateContainer] = ()
    typedOtherAttributes: List[TypedAttributeUpdateContainer] = ()


@dataclass
class TypedAttributeUpdateContainer(TcBaseObj):
    """
    Updates for a certain type of object.
    
    :var objectType: The object type containing the attributes.
    :var updates: The list of attributes and their values.
    """
    objectType: str = ''
    updates: List[AttributeUpdateContainer] = ()


@dataclass
class CreatedDependenciesContainer(TcBaseObj):
    """
    New  dependencies container
    
    :var createdDependencies: The dependencies created
    :var createdProxyTasks: The created proxy tasks
    :var updatedTasks: The tasks which were updated due to
    :var serviceData: The SOA service data
    """
    createdDependencies: List[TaskDependency] = ()
    createdProxyTasks: List[Fnd0ProxyTask] = ()
    updatedTasks: List[ScheduleTask] = ()
    serviceData: ServiceData = None


@dataclass
class CreatedObjectsContainer(TcBaseObj):
    """
    The container of created objects
    
    :var createdObjects: The objects which were created.
    :var updatedTasks: The tasks affected by the creation.
    :var serviceData: The service data.
    """
    createdObjects: List[POM_object] = ()
    updatedTasks: List[ScheduleTask] = ()
    serviceData: ServiceData = None


@dataclass
class CriticalTasksContainer(TcBaseObj):
    """
    The tasks on the critical path.
    
    :var tasks: The tasks on the critical path.
    :var proxyTasks: The proxy tasks on the critical path.
    :var serviceData: The service data.
    """
    tasks: List[ScheduleTask] = ()
    proxyTasks: List[Fnd0ProxyTask] = ()
    serviceData: ServiceData = None


@dataclass
class DeleteTaskContainer(TcBaseObj):
    """
    Information about the deleted tasks.
    
    :var orphaned: Tasks which were orphaned(as they could not be deleted).
    :var updated: Tasks updated due to the delete.
    :var serviceData: The service data.
    """
    orphaned: List[ScheduleTask] = ()
    updated: List[ScheduleTask] = ()
    serviceData: ServiceData = None


@dataclass
class DependencyCreateContainer(TcBaseObj):
    """
    Creates Dependencies between tasks in the same schedule, between a task and a proxy task in the same schedule, or
    between a tasks in different schedules (but in the same master schedule).  It returns the created dependencies,
    created proxy tasks (if any), and the objects affected by this change
    
    :var predTask: The predecessor task for the dependency.
    :var succTask: The successor task for the dependency.
    :var depType: The type of dependency: 0-FS, 1-FF, 2-SS, 3-SF
    :var lagTime: The lag time in minutes (480 ~ 1 day)
    """
    predTask: WorkspaceObject = None
    succTask: WorkspaceObject = None
    depType: int = 0
    lagTime: int = 0


@dataclass
class DependencyUpdateContainer(TcBaseObj):
    """
    Update dependency container
    
    :var dependency: The dependency to update.
    :var newType: The new type for the dependency.
    :var newLag: The new lag for the dependnecy.
    """
    dependency: TaskDependency = None
    newType: int = 0
    newLag: int = 0



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.ImportExport._2011_06.FileImportExport import FileMetaData, ImportExportOptions
from typing import List
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ImportFromApplicationInputData3(TcBaseObj):
    """
    Structure represents the parameters required to import a requirement specification.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this input structure.
    :var transientFileWriteTicket: Transient File write ticket for the file to be imported.
    :var applicationFormat: Application format associated with the file to be imported into system.
    :var createSpecElementType: Subtype of specification element to be imported.
    :var specificationType: Type of the specification to be imported. RequirementSpec is default.
    :var isLive: Flag to determine live or non live import.
    :var selectedBomLine: BOMLine under which new structure will get imported.
    :var fileMetaDatalist: List of FileMetaData that has the imported file information. One for each imported
    specification.
    :var importOptions: List of options for import such as keywords. One for each imported specification.
    :var specDesc: Description to be set for an Item.
    """
    clientId: str = ''
    transientFileWriteTicket: str = ''
    applicationFormat: str = ''
    createSpecElementType: str = ''
    specificationType: str = ''
    isLive: bool = False
    selectedBomLine: BusinessObject = None
    fileMetaDatalist: List[FileMetaData] = ()
    importOptions: List[ImportExportOptions] = ()
    specDesc: str = ''



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMLine
from tcsoa.gen.Cae._2014_06.StructureManagement import DerivativeRuleResponse
from tcsoa.base import TcService


class StructureManagementService(TcService):

    @classmethod
    def executeDeriveCAEStructure(cls, rootBOMLine: BOMLine, selectedRule: str, optionalRootName: str, optionalIndex: str) -> DerivativeRuleResponse:
        """
        This operation creates a cloned/variant structure from the root BOMLine of a BOM structure loaded in a model
        BOMWindow and a derivative rule name to be applied. The output BOM structure is determined by the XSLT-based
        clone rules executed against the input BOM structure. These clone rules are a part of the derivative rule
        configuration. Several clone rules are defined under a derivative rule. There can be several derivative rules
        defined in the database by Simulation Administrator. 
        
        Given an input BOM structure, the derivative rules is applied to the BOM structure and generate a corresponding
        output structure. The output is a clone/variant based on the clone rules defined in the derivative rule
        configuration.
        
        The output structure may optionally have a user defined name for the root, suffixed with a user defined
        numerical index. 
        
        This operation uses the existing Data Mapping functionality to apply the clone rules against input structure
        and generate the output structure. Data mapping defines the mapping between an input item type and its
        resulting output item type. The Derive CAE Structure operation specific data mapping rules will co-exist with
        existing configurations, stored in the datamapping.xml file. This file is maintained in the database. 
        
        To use Derive CAE Structure operation, well-defined CAEDerivativeRuleConfig.xml, datamapping.xml and
        NodeConfigXML.xml files are required in the database.
         CAEDerivativeRuleConfig.xml file contains the configurations and conditional clone rules pertaining to the
        Derivative Rule . Datamapping.xml contains mapping rules configured for mapping between input and output CAE
        Structures. NodeXMLconfig.xml is has configured item attributes, which are mapped between input and output CAE
        Structure. 
        Simulation Administrators or DBAs should assign the location for CAEDerivativesRuleConfig.xml using Derivative
        Rules user preferences. Derivative Rules view is used to create a Derivative Rule.
        Similarly, Datamapping.xml and NodeXMLConfig.xml  are also configured by an Administrator and stored at the
        location in database which is pointed by user preference CAE_datamap_files_location.
        
        
        Use cases:
        1: Create an output structure given a top BOMLine of the input structure, a Derivative Rule name.
        
        Given an input BOM structure, this operation will apply the clone rules contained within the specified
        Derivative Rule configuration to the BOM structure and generate a corresponding output structure. 
        The output structure would be an exact clone/variant based on the rules defined in the configuration.
        """
        return cls.execute_soa_method(
            method_name='executeDeriveCAEStructure',
            library='Cae',
            service_date='2014_06',
            service_name='StructureManagement',
            params={'rootBOMLine': rootBOMLine, 'selectedRule': selectedRule, 'optionalRootName': optionalRootName, 'optionalIndex': optionalIndex},
            response_cls=DerivativeRuleResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMLine, ImanRelation
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class InvalidAssociation(TcBaseObj):
    """
    Invalid Association data
    
    :var associationType: Association Type
    :var primary: primary BOMLine
    :var secondary: secondary BOMLine.
    :var relation: Invalid relation object
    :var reason: Reason why relation is invalid
    """
    associationType: str = ''
    primary: BOMLine = None
    secondary: BOMLine = None
    relation: ImanRelation = None
    reason: str = ''


@dataclass
class ValidateInStructureAssociationsResponse(TcBaseObj):
    """
    Contains the Service data and array of invalid associations data
    
    :var serviceData: This is a common data structure used to return sets of Teamcenter Data Model object from a
    service request. This also holds services exceptions.
    :var failedAssociations: Vector of invalid associations
    """
    serviceData: ServiceData = None
    failedAssociations: List[InvalidAssociation] = ()



from __future__ import annotations

from tcsoa.gen.Internal.Administration._2012_10.OrganizationManagement import GetOrganizationUserMembersResponse, OrganizationMembersInput
from tcsoa.base import TcService


class OrganizationManagementService(TcService):

    @classmethod
    def getOrganizationGroupMembers(cls, input: OrganizationMembersInput) -> GetOrganizationUserMembersResponse:
        """
        This operation searches for GroupMember objects in Teamcenter organization. The found GroupMember objects would
        have either user ID or user name matches with the user ID and user name in search criteria, and group name and
        role name match the group name and role name respectively in search criteria. It also has option to return
        inactive GroupMember objects.
        
        Use cases:
        Find group members for given group, role, user id and user name.
        """
        return cls.execute_soa_method(
            method_name='getOrganizationGroupMembers',
            library='Internal-Administration',
            service_date='2012_10',
            service_name='OrganizationManagement',
            params={'input': input},
            response_cls=GetOrganizationUserMembersResponse,
        )



from __future__ import annotations

from tcsoa.gen.Internal.DebugMonitor._2014_06.JournalBasedTesting import AuxiliaryInfo
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class TerminateJBTResponse(TcBaseObj):
    """
    Contains the service data and any additional information such as the mark point rollback status.
    
    :var additionalInfo: The additional information about a test run. Currently it returns the mark point rollback
    status in the strToStrVectorMap: key is "rollback" and value is "passed" or "failed".
    :var serviceData: The service data containing partial errors.
    """
    additionalInfo: AuxiliaryInfo = None
    serviceData: ServiceData = None



from __future__ import annotations

from enum import Enum
from typing import Dict, List
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class MeasuredValue(TcBaseObj):
    """
    The measurement value for a metric and some associated information.
    
    :var dType: The data type of the value.
    :var value: Measurement value in a measurement unit relevant to the metric.
    """
    dType: str = ''
    value: str = ''


@dataclass
class MeasurementInfo(TcBaseObj):
    """
    Measurement data for each requested performance metric.
    
    :var timeStamp: The time at which the measurement was recorded.
    :var measvals: The measurement value and some associated information.
    :var additionalInfo: A place holder for data recorded in addition to measurement value. For example, the name of
    operation may be recorded as additional information with the SqlTotalTime metric.
    """
    timeStamp: datetime = None
    measvals: MeasuredValue = None
    additionalInfo: List[str] = ()


@dataclass
class MetaData(TcBaseObj):
    """
    It represents the vector containing list of meta data for various metrics. A collection of characteristics or
    properties of metrics.
    
    :var metricId: A unique identifier that can identify a particular metric from a list of performance metrics.
    :var dType: The data type for the measured value. The valid values are MetricDataType_Integer and
    MetricDataType_Double.
    :var displayName: User friendly name for display on UI.
    :var metricMode: The metric monitoring mode. Possible values are - Off, Collect or Alert.
    :var unitOfMeasurement: The measurement unit for the values being measured. For a time based metric it could be
    "seconds".
    :var maxEntriesToKeep: The total number of measurement values to be kept in history for a particular metric. If the
    numbers of values exceed this value the less important measured value is deleted first.
    :var metricThreshold: Collected values exceeding this threshold will result in the Teamcenter Server Health
    Monitoring System generating an Alert for this metric.
    :var monStartTime: The time at which the monitoring is started for the particular metric.
    :var metricType: Metrics can be classified based on the method of monitoring. 'metricType' denotes which method is
    adopted for monitoring.
    """
    metricId: str = ''
    dType: str = ''
    displayName: str = ''
    metricMode: MetricMode = None
    unitOfMeasurement: str = ''
    maxEntriesToKeep: int = 0
    metricThreshold: Threshold = None
    monStartTime: datetime = None
    metricType: MetricType = None


@dataclass
class MetricAttributes(TcBaseObj):
    """
    A performance metric identifier and the time interval for which the data should be collected.
    
    :var metricId: Uniquely identifies a particular metric from a list of performance metrics. The entire list of valid
    metric IDs can be found in serverMonitorConfig.xml in TC_DATA folder.
    :var timeInterval: The time duration(seconds) for which measurements will be collected. A value of 30 would collect
    the last 30 seconds of measurement data. A value of 0 results in the full measurement history being returned.
    """
    metricId: str = ''
    timeInterval: int = 0


@dataclass
class MetricConfig(TcBaseObj):
    """
    It holds the characteristics relevant to a metric. These include the monitoring mode of metric, the maximum number
    of measurements to be kept for the metric, the threshold value and the threshold period.
    
    :var mode: It signifies the mode of monitoring i. e. Off, Collect or Alert.
    :var numMaxEntriesToKeep: Maximum number of measurements to be stored for this metric.
    :var thresholdValue: The value exceeding which the Teamcenter Health Monitoring System will generate alert if it is
    in Alert mode.
    :var thresholdPeriod: The period during which the threshold value needs to be monitored for generating alerts.
    """
    mode: MetricMode = None
    numMaxEntriesToKeep: int = 0
    thresholdValue: str = ''
    thresholdPeriod: int = 0


@dataclass
class MetricMetaData(TcBaseObj):
    """
    It represents the vector containing list of meta data for various metrics. A collection of characteristics or
    properties of metrics.
    
    :var metaData: A collection of characteristics or properties of metrics.
    """
    metaData: List[MetaData] = ()


@dataclass
class OperationalHealthData(TcBaseObj):
    """
    Measurement data for each requested performance metric.
    
    :var measurementInfoMap: A map which holds the list of measurements monitored associated with the metric. A map of
    metric IDs and measurement data ( string / EventStatistics ).
    """
    measurementInfoMap: MeasurementInfoMap = None


@dataclass
class Threshold(TcBaseObj):
    """
    It holds the threshold information of Teamcenter Server Health metrics.  When a threshold is exceeded for a metric
    the Teamcenter Health Monitoring System may generate alerts.
    
    :var thresholdValue: The boundary value for which the Teamcenter Server Health Monitoring System may generate
    alerts for this metric.
    :var hasPeriod: A true value signifies that the threshold has a time period associated with it.
    :var thresholdPeriod: The time period (seconds) associated with threshold. It is meaningful for those metrics which
    also consider time period for monitoring like CountableMetricWithTimePeriod.
    """
    thresholdValue: str = ''
    hasPeriod: bool = False
    thresholdPeriod: int = 0


@dataclass
class ValidMetrics(TcBaseObj):
    """
    A list of valid metric IDs.
    
    :var metricIds: List of all metricIds.
    """
    metricIds: List[str] = ()


@dataclass
class AlertData(TcBaseObj):
    """
    This holds the alert data associated with the metric. The information includes the subject of generated alert, the
    summary of the alert and event data causing the alert.
    
    :var subject: Subject line that contains information regarding which metric generated alert etc.
    :var summary: A summary containing information about the set threshold and the value that exceeded threshold.
    :var eventData: Measured event values for the metric.
    """
    subject: str = ''
    summary: str = ''
    eventData: List[EventData] = ()


@dataclass
class AlertEventData(TcBaseObj):
    """
    Measurement data for each requested metric.
    
    :var alertData: A map of metric IDs and alert data ( string / AlertData ).
    """
    alertData: AlertDataMap = None


@dataclass
class EventData(TcBaseObj):
    """
    This holds the information about one event which includes date and time, value of the measurement, and vector of
    additional information. Measured event values for the metric.
    
    :var time: The time at which the measurement was taken.
    :var value: Measurement value recorded.
    :var additionalInfo: A place holder for data recorded in addition to measurement value. For example, the name of
    operation may be recorded as additional information with the SqlTotalTime metric
    """
    time: str = ''
    value: str = ''
    additionalInfo: List[str] = ()


@dataclass
class EventStatistics(TcBaseObj):
    """
    It holds the event data associated with the given metrics for the specified time interval. The information includes
    a list of measurements and event statistics.
    
    :var measInfo: The measurement statistics for the given time period.
    :var eventStats: Some statistics based on the type of metric. 
    - For a CountableMetric or CountableMetricWithTimePeriod this would be number of events that have taken place since
    monitoring started. 
    - For VariableMetric or AvgVariableWithTimePeriod  it would be average value of measurements recorded so far. 
    - In case of MaxVariableWithTimePeriod, it would be the maximum measurement value recorded in the measurement
    history of that metric.
    -  For DetailedMetric, it would be average time taken per SQL call for an operation.
    
    """
    measInfo: List[MeasurementInfo] = ()
    eventStats: str = ''


class MetricMode(Enum):
    """
    A metric can operate in any of the three different monitoring modes Off, Collect or Alert. This method retrieves
    the current mode of operation. When in Off mode, the monitoring system does not record any monitoring data for the
    performance metrics. Since, there is no history maintained no alerts situation happens. In the Collect mode, 
    measurement data is recorded but no alerts are generated when thresholds exceed. In the Alert mode, the measurement
    data is not only collected but alerts are generated when thresholds for any of the metrics is exceeded
    """
    Off = 'Off'
    Collect = 'Collect'
    Alert = 'Alert'


class MetricType(Enum):
    """
    Type of the metric.
    - CountableMetric A metric for which the number or count of events is of importance and is considered for alert
    generation. If the threshold value is 'n' it means the alert will be generated if event has occurred 'n' number of 
        times or more since its monitoring started.
    - CountableMetricWithTimePeriod     Same as CountableMetric, but the count is over an elapsed period of time.
    - VariableMetric     For a VariableMetric, the measured value is of significance and an alert will be triggered if
    the measured value for a metric equals or exceeds the threshold value. An alert is triggered when a measured value
    is equal to or exceeds the threshold.
    - AvgVariableWithTimePeriod    An Alert is triggered when the running average of the measured value is equal to or
    exceeds the threshold.
    - MaxVariableWithTimePeriod    An alert is triggered when the measured value is equal to or exceeds the threshold
    value over a given time period.    
    - DetailedMetric    This is a special type of metric specific to SQL statistics which does not result in any alert
    generation by Teamcenter Server Health Monitoring System. The average time taken per SQL call is of significance in
    its context.
    
    """
    CountableMetric = 'CountableMetric'
    CountableMetricWithTimePeriod = 'CountableMetricWithTimePeriod'
    VariableMetric = 'VariableMetric'
    AvgVariableWithTimePeriod = 'AvgVariableWithTimePeriod'
    MaxVariableWithTimePeriod = 'MaxVariableWithTimePeriod'
    DetailedMetric = 'DetailedMetric'


"""
A Map which holds the list of measurements monitored associated with the metric. A map of metric IDs and measurement data ( string / EventStatistics ).
"""
MeasurementInfoMap = Dict[str, List[EventStatistics]]


"""
A Map that holds metric Configuration information. A map of metric IDs and metric configuration data ( string / MetricConfig ).
"""
MetricConfigMap = Dict[str, MetricConfig]


"""
A Map which holds the alert data associated with the metric. A map of metric IDs and alert data ( string / AlertData ).
"""
AlertDataMap = Dict[str, AlertData]



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GenerateScopedMultipleStructure3Response(TcBaseObj):
    """
    GenerateScopedMultipleStructure3Response struct
    
    :var ticket: The transient file ticket to be used for downloading the generated plmxml
    :var fileTickets: The transient file tickets for any files exported during generation of plmxml
    :var relativeFolderName: Name of the folder where the transient fileTickets have to be downloaded relative to the
    folder where plmxml file is downloaded.
    :var fileNames: filenames as they appear in the transient volume under the plmxml created folder.
    :var data: partial failures are returned - along with object ids for each plmxml data could not be generated.
    """
    ticket: str = ''
    fileTickets: List[str] = ()
    relativeFolderName: str = ''
    fileNames: List[str] = ()
    data: ServiceData = None



import enum


class EPMAttachmentType(enum.IntEnum):
    """ Necessary when using WorkflowService#createInstance or WorkflowService#addAttachments """

    EPM_target_attachment = 1
    EPM_reference_attachment = 3
    EPM_signoff_attachment = 4
    EPM_release_status_attachment = 5
    EPM_comment_attachment = 6
    EPM_instruction_attachment = 7
    EPM_interprocess_task_attachment = 8
    EPM_schedule_task_attachment = 9
    EPM_replica_target_attachment = 10
    EPM_user_attachment = 1000


class EPMAttachmentType2(enum.Enum):
    """ Necessary when using WorkflowService.createWorkflow or WorkflowService#createRemoteWorkflowAsync """

    EPM_ATT_TARGET = "target"
    EPM_ATT_REFERENCE = "reference"
    EPM_ATT_PROCESS = "process"
    EPM_ATT_SIGNOFF = "signoff"
    EPM_ATT_COMMENT = "comment"
    EPM_ATT_INSTRUCTION = "instruction"
    EPM_ATT_INTERPROCESS_TASK = "interprocess_task"
    EPM_ATT_SCHEDULE_TASK = "schedule_task"
    EPM_ATT_BOTH = "both"
    EPM_ATT_ALL = "all"


class EPMConditionTask(enum.IntEnum):
    EPM_RESULT_FALSE = 0
    EPM_RESULT_TRUE = 1
    EPM_RESULT_UNSET = 2


class RelationTypes(enum.Enum):
    EPM_TEMPLATE_BASED_ON_RELATION_TYPE = "EPM_template_based_on"
    EPM_SIGNOFF_PROFILE_RELATION_TYPE = "EPM_signoff_profile"
    SUBSCRIBED_REMOTEINBOXES_RELATION_TYPE = "subscribed_remoteinboxes"



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from enum import Enum
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class FileTicket(TcBaseObj):
    """
    FileTicket details
    
    :var ticket: the FMS ticket
    :var originalFileName: name of the file that is displayed in TC UI
    """
    ticket: str = ''
    originalFileName: str = ''


@dataclass
class ApplicationRef(TcBaseObj):
    """
    Structure representing application neutral identifier
    
    :var application: The application where these fields can be resolved. For example - it is Teamcenter for TC.
    :var label: The identifier for the object.
    :var version: The optional version string.
    """
    application: str = ''
    label: str = ''
    version: str = ''


@dataclass
class GenerateFullSyncRequestResponse(TcBaseObj):
    """
    Used to generate the plmxml for a sync request.
    
    :var ticket: ticket
    :var data: data
    """
    ticket: FileTicket = None
    data: ServiceData = None


@dataclass
class GenerateStructureResponse(TcBaseObj):
    """
    generatestructure returns the transient file ticket on success.
    
    :var ticket: ticket
    :var data: data
    """
    ticket: str = ''
    data: ServiceData = None


@dataclass
class GetFileReadTicketsResponse(TcBaseObj):
    """
    tickets to download files referenced by plmxml.
    
    :var tickets: tickets
    :var data: data
    """
    tickets: List[FileTicket] = ()
    data: ServiceData = None


@dataclass
class GetFileWriteTicketsResponse(TcBaseObj):
    """
    Tickets to upload files referenced by plmxml.
    
    :var tickets: tickets
    :var data: data
    """
    tickets: List[str] = ()
    data: ServiceData = None


@dataclass
class GetNextApprovedRequestResponse(TcBaseObj):
    """
    get next approved request. Currently not used.
    
    :var ticket: ticket
    :var data: data
    """
    ticket: FileTicket = None
    data: ServiceData = None


@dataclass
class GetPropertiesOfObjectsResponse(TcBaseObj):
    """
    GetPropertiesOfObjectsResponse struct
    
    :var nameValues: nameValues
    :var data: data
    """
    nameValues: StrVStrMap = None
    data: ServiceData = None


@dataclass
class GetStructureReadTicketResponse(TcBaseObj):
    """
    response of getStructureReadTicket - returns the plmxml file ticket.
    
    :var ticket: ticket
    :var data: data
    """
    ticket: FileTicket = None
    data: ServiceData = None


@dataclass
class GetStructureWriteTicketResponse(TcBaseObj):
    """
    response of getStructureWriteTicket - returns the ticket to be used for uploading a plmxml file.
    
    :var ticket: ticket
    :var data: data
    """
    ticket: str = ''
    data: ServiceData = None


@dataclass
class ObjectsExistResponse(TcBaseObj):
    """
    response of ObjectsExist method
    
    :var bExist: does the object exist in Teamcenter or not.
    :var ids: All the ApplicationRef registered for the Teamcenter object
    :var data: used to report any partial failures.
    """
    bExist: List[bool] = ()
    ids: List[ApplicationRef] = ()
    data: ServiceData = None


@dataclass
class ProjectCreationData(TcBaseObj):
    """
    Project(AI) creation method data
    
    :var type: Type of the Application Interface object to be created.
    :var name: Name of the AI object to be created.
    :var description: Description of the AI object to be created.
    :var targetApplicationId: The application id to be set on the AI object.
    """
    type: str = ''
    name: str = ''
    description: str = ''
    targetApplicationId: str = ''


@dataclass
class ProjectFilter(TcBaseObj):
    """
    Structure to specify the filter when using GetProjects method.
    
    :var projectType: ProjectType enum. Set it to ProjectType_Any if not needed.
    :var name: name of the AppliationInterface Object
    :var releasedBefore: filtering by Date
    :var releasedAfter: filtering by Date
    :var applicationId: maps to the Export TransferMode's (of the AI) context string
    :var siteId: description of the ApplicationInterface Object
    :var targetAppProjectId: used only if projectType==ProjectType_Existing
    :var description: description of the ApplicationInterface Object
    :var type: type of the ApplicationInterface Object. The type must be a valid type of AI Object.
    :var userId: userId to filter on
    :var groupName: filter on AI objects using groupName
    :var createdBefore: filtering by Date
    :var createdAfter: filtering by Date
    :var modifiedBefore: filtering by Date
    :var modifiedAfter: filtering by Date
    """
    projectType: ProjectType = None
    name: str = ''
    releasedBefore: datetime = None
    releasedAfter: datetime = None
    applicationId: str = ''
    siteId: str = ''
    targetAppProjectId: str = ''
    description: str = ''
    type: str = ''
    userId: str = ''
    groupName: str = ''
    createdBefore: datetime = None
    createdAfter: datetime = None
    modifiedBefore: datetime = None
    modifiedAfter: datetime = None


@dataclass
class ProjectInfo(TcBaseObj):
    """
    Structure to specify ApplicationInterface information.
    
    :var targetAppProjectId: The projectId string recorded on the ApplicationInterface Object
    :var name: name of the AppliationInterface Object
    :var description: description of the ApplicationInterface Object
    """
    targetAppProjectId: str = ''
    name: str = ''
    description: str = ''


@dataclass
class RequestDetail(TcBaseObj):
    """
    Structure representing the details of the RequestObject
    
    :var name: name of the RequestObject
    :var description: description of the RequestObject
    :var stateDesc: description on the state of the RequestObject.
    :var status: the status fields of the RequestObject
    :var scope: RequestScope_Whole - no ExternalReference elements will be found in plmxml. If Partial then there will
    be ExternalReference elements in plmxml.
    :var update: used to specify an incremental update.
    """
    name: str = ''
    description: str = ''
    stateDesc: str = ''
    status: StatusInfo = None
    scope: RequestScope = None
    update: UpdateType = None


@dataclass
class StateFilter(TcBaseObj):
    """
    Structure to filter RequestObject
    
    :var states: vector of RequestState enums
    :var types: vector of RequestType enums
    """
    states: List[RequestState] = ()
    types: List[RequestType] = ()


@dataclass
class StatusInfo(TcBaseObj):
    """
    Structure representing the status fields of the RequestObject
    
    :var status: RequestStatus enum RequestStatus_Normal etc.
    :var description: status message of the Request.
    """
    status: RequestStatus = None
    description: str = ''


@dataclass
class CommitFileData(TcBaseObj):
    """
    commitFileData structure to be used when commiting files to teamcenter.
    
    :var ticket: the write ticket returned from Teamcenter from a prior call to GetWriteFileTickets.
    :var id: If used with AI object, this represents the id of the RequestObject.
    :var filename: filename as specified, when getting the write ticket.
    """
    ticket: str = ''
    id: ApplicationRef = None
    filename: str = ''


@dataclass
class CommitFilesResponse(TcBaseObj):
    """
    commit files response.
    
    :var failedCommits: failedCommits
    :var data: data
    """
    failedCommits: List[CommitFileData] = ()
    data: ServiceData = None


@dataclass
class CommitStructureFileResponse(TcBaseObj):
    """
    returns the fileIds if the saved plmxml file.
    
    :var fileId: fileId
    :var data: data
    """
    fileId: str = ''
    data: ServiceData = None


@dataclass
class Configuration(TcBaseObj):
    """
    Configuration structure.
    
    :var id: Tag of ConfigurationContext or StructureContext or RevRule or VariantRule or storedOptionSetId,or
    TransferMode.
    :var rulename: if id is NULLTAG, then used to specify the revisionrule by name.
    :var useDefaultRevisionRule: if true - the Teamcenter preferences are used to pick up the rev rule. Overrides the
    above 2 members if true.
    """
    id: BusinessObject = None
    rulename: str = ''
    useDefaultRevisionRule: bool = False


@dataclass
class CreatePublishRequestResponse(TcBaseObj):
    """
    response of the createPublishRequest method.
    
    :var ticket: ticket
    :var data: data
    """
    ticket: FileTicket = None
    data: ServiceData = None


class FileType(Enum):
    """
    Type of file being uploaded to Teamcenter.
    """
    Binary = 'FileType_Binary'
    Text = 'FileType_Text'


class ProjectType(Enum):
    """
    Filter on whether the project_id is set on the ApplicationInterface object or not
    """
    New = 'ProjectType_New'
    Existing = 'ProjectType_Existing'
    Any = 'ProjectType_Any'


class RequestScope(Enum):
    """
    RequestScope
    """
    Whole = 'RequestScope_Whole'
    Partial = 'RequestScope_Partial'
    Any = 'RequestScope_Any'


class RequestState(Enum):
    """
    State of the Request Object
    """
    New = 'RequestState_New'
    Processing = 'RequestState_Processing'
    Pending = 'RequestState_Pending'
    Communicating = 'RequestState_Communicating'
    Completed = 'RequestState_Completed'
    Rejected = 'RequestState_Rejected'
    Any = 'RequestState_Any'


class RequestStatus(Enum):
    """
    Status of the Request Object. TC or application can set these depending on which party is doing the authoring.
    """
    Normal = 'RequestStatus_Normal'
    Warning = 'RequestStatus_Warning'
    Severe = 'RequestStatus_Severe'
    Abort = 'RequestStatus_Abort'
    Any = 'RequestStatus_Any'


class RequestType(Enum):
    """
    Type of the Request Object
    """
    Publish = 'RequestType_Publish'
    Sync = 'RequestTypeSync'
    Any = 'RequestType_Any'
    Notify = 'RequestType_Notify'


class UpdateType(Enum):
    """
    Request update type
    """
    Full = 'UpdateType_Full'
    Delta = 'UpdateType_Delta'
    Any = 'UpdateType_Any'


"""
StrVStrMap
"""
StrVStrMap = Dict[str, List[str]]



from __future__ import annotations

from tcsoa.gen.Internal.Administration._2014_10.UserManagement import ActivateUserInput
from tcsoa.gen.Internal.Administration._2013_05.UserManagement import LicenseStatusResponse
from typing import List
from tcsoa.base import TcService


class UserManagementService(TcService):

    @classmethod
    def activateUsers2(cls, activateUsers: List[ActivateUserInput]) -> LicenseStatusResponse:
        """
        This operation activates user(s) based on the number of allowed active author, consumer, or occasional user
        licenses,  associated with license bundles in a license server. If enough licenses are not available for either
        the license level or the license bundle, a partial error is returned with the index of the activateUsers input.
        """
        return cls.execute_soa_method(
            method_name='activateUsers2',
            library='Internal-Administration',
            service_date='2014_10',
            service_name='UserManagement',
            params={'activateUsers': activateUsers},
            response_cls=LicenseStatusResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class FlowInput(TcBaseObj):
    """
    FlowInput
    
    :var successor: The successor object of the flow object
    :var predecessor: The predecessor object of the flow.
    :var delay: The delay time for the successor.
    """
    successor: BusinessObject = None
    predecessor: BusinessObject = None
    delay: float = 0.0


@dataclass
class FlowResponse(TcBaseObj):
    """
    FlowResponse
    
    :var flowsResult: A vector of the new succesfully created flow objects.
    :var serviceData: serviceData
    """
    flowsResult: List[BusinessObject] = ()
    serviceData: ServiceData = None


@dataclass
class GetResolvedNodesFromLAResponse(TcBaseObj):
    """
    Response for getResolvedNodesFromLA SOA
    
    :var outResolvedNodes: All the resolved nodes of all objects
    :var serviceData: Service data object
    """
    outResolvedNodes: List[ResolvedNodesOutput] = ()
    serviceData: ServiceData = None


@dataclass
class LogicalAssignmentAttribute(TcBaseObj):
    """
    Attribute representation
    
    :var name: Attribute name
    :var value: Attribute value
    """
    name: str = ''
    value: str = ''


@dataclass
class LogicalAssignmentData(TcBaseObj):
    """
    Parameters for editing Logical Assignment object
    
    :var laObj: Logical Assignment object to edit
    :var laAttributes: Attributes to set to the object
    """
    laObj: BusinessObject = None
    laAttributes: List[LogicalAssignmentAttribute] = ()


@dataclass
class LogicalAssignmentResolvedNodes(TcBaseObj):
    """
    Holds resolved nodes of each LA
    
    :var laObj: LA object
    :var resolvedNodes: Resolved nodes of LA object
    """
    laObj: BusinessObject = None
    resolvedNodes: List[BusinessObject] = ()


@dataclass
class ResolveData(TcBaseObj):
    """
    Data for object to resolve
    
    :var object: object to resolve
    :var laResolveType: what type of LAs to resolve (class name)
    :var recursive: if this flag is true, resolve all LAs of the above type of this object, all operations and
    processes under this object, all operations and processes under child processes and so on
    """
    object: BusinessObject = None
    laResolveType: str = ''
    recursive: bool = False


@dataclass
class ResolvedNodesInput(TcBaseObj):
    """
    Input for getResolvedNodesFromLA SOA
    
    :var parentObject: parent operation/process
    :var laObjects: vector of LA objects of this op/proc, for which we want resolved nodes
    """
    parentObject: BusinessObject = None
    laObjects: List[BusinessObject] = ()


@dataclass
class ResolvedNodesOutput(TcBaseObj):
    """
    Holds resolved nodes of the whole parent object
    
    :var parentObject: Parent object
    :var laResolvedNodes: Resolved nodes of all LA objects
    """
    parentObject: BusinessObject = None
    laResolvedNodes: List[LogicalAssignmentResolvedNodes] = ()


@dataclass
class CalculateCriticalPathResponse(TcBaseObj):
    """
    The response structure for the calculateCriticalPath operation.
    
    :var results: A list of CalculateCriticalPathResult structures for each path returned.
    :var serviceData: This is a common data strucuture used to return sets of
    Teamcenter Data Model object from a service request. This also holds services exceptions.
    """
    results: List[CalculateCriticalPathResult] = ()
    serviceData: ServiceData = None


@dataclass
class CalculateCriticalPathResult(TcBaseObj):
    """
    A structure that represents a path returned by the computeCriticalPath operation
    
    :var object: The root object for which this path was computed
    :var components: A list of components that make up the path. This includes the connecting flows.
    :var duration: The total duration of the path.
    """
    object: BusinessObject = None
    components: List[BusinessObject] = ()
    duration: float = 0.0



from __future__ import annotations

from typing import List
from tcsoa.gen.BusinessObjects import WorkspaceObject, PSBOMViewRevision
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class FindHighestFindNumInExpandInput(TcBaseObj):
    """
    Input structure for findHighestFindNumberInExpand operation
    
    :var clientId: Identifier that helps the client to track the object(s) created.
    :var bvr: bvr for which the highest sequence number is expected to be returned
    :var occType: (optional) The name of the occurrence type. If the occurrence type is set here, 
    the highest sequence number is only estimated from expanded BOMLines with specific occurrence type. If the
    occurrence type is an empty string, all expanded BOMLines are used to get the highest sequence number.
    """
    clientId: str = ''
    bvr: PSBOMViewRevision = None
    occType: str = ''


@dataclass
class FindHighestFindNumberInExpandResponse(TcBaseObj):
    """
    Return structure for findHighestFindNumberInExpand operation
    
    :var incrementNumber: stride length between find numbers
    :var serviceData: Standard ServiceData member
    :var output: vec of output structures
    """
    incrementNumber: int = 0
    serviceData: ServiceData = None
    output: List[FindHighestFindNumberOutputStruct] = ()


@dataclass
class FindHighestFindNumberOutputStruct(TcBaseObj):
    """
    Return structure for findHighestFindNumberOutputStruct
    
    :var highestFindNumber: The highest find number
    :var clientId: client id
    """
    highestFindNumber: int = 0
    clientId: str = ''


@dataclass
class CopyRecursivelyResponse(TcBaseObj):
    """
    This structure provides a set of input values for the re-sequence action.
    
    :var createdIcRevs: any newly created IncrementalChange revisions
    :var createdFutureIcRevs: any newly created future IC revisions.
    :var serviceData: the newly created object will be in here
    """
    createdIcRevs: List[WorkspaceObject] = ()
    createdFutureIcRevs: List[WorkspaceObject] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.base import TcService


class PreferenceManagementService(TcService):

    @classmethod
    def lockSitePreferences(cls) -> bool:
        """
        Locks the Site preferences stored in the database. 
        
        This can be used by system administrators only. It is not mandatory to lock Site preferences to make changes,
        but it ensures exclusive write accesses when necessary.
        
        Exceptions:
        >The exception will not contain any error code.
        """
        return cls.execute_soa_method(
            method_name='lockSitePreferences',
            library='Administration',
            service_date='2008_06',
            service_name='PreferenceManagement',
            params={},
            response_cls=bool,
        )

    @classmethod
    def unlockSitePreferences(cls) -> bool:
        """
        Releases the lock set on the site preferences stored in the database. The locking comes from the call to
        lockSitePreferences operation. Only the user who locked the site preferences is allowed to unlock them. As of
        Teamcenter 11.6, due to the re-architecture of preferences from XML storage to database objects, this operation
        is no longer required. So it is deprecated.
        
        Exceptions:
        >The exception will not contain any error code:
        """
        return cls.execute_soa_method(
            method_name='unlockSitePreferences',
            library='Administration',
            service_date='2008_06',
            service_name='PreferenceManagement',
            params={},
            response_cls=bool,
        )



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetVendorCatalogInfo2Response(TcBaseObj):
    """
    Contains a ServiceData object that may contain error descriptions, if any errors occurred, and a list of catalog
    information objects providing detailed information about Generic Tool Catalog vendor catalogs that are stored on
    the Teamcenter server.
    
    :var serviceData: The service data
    :var catalogInfo: The vendor catalog information
    """
    serviceData: ServiceData = None
    catalogInfo: List[CatalogInfo2] = ()


@dataclass
class CatalogInfo2(TcBaseObj):
    """
    The structure contains vendor catalog information 
    
    
    :var vendorName: The name of the tool vendor company providing the catalog
    :var vendorAcronym: An acronym for the tool vendor providing the catalog, usually consisting of two or three
    characters 
    :var gtcHierarchyVersion: The Generic Tool Catalog version number, in the format:VR, for example "V1R7"
    :var gtcPackageId: A unique identifier for this Generic Tool Catalog package
    :var vendorCatalogVersion: The version of the catalog
    :var vendorCatalogLanguage: The language of the catalog
    :var vendorCatalogDescription: A long description of the catalog
    :var vendorCatalogShortDescription: A short description of the catalog
    :var vendorCatalogId: A unique ID for the catalog
    :var vendorCatalogRootClassId: The ID of the catalog's root class in the Generic Tool Catalog's Classification
    hierarchy when the catalog gets imported, typically the concatenation of the vendor's acronym and "_GTC"
    :var vendorCatalogRootDir: The root directory where the tool vendor catalog is stored on the Teamcenter server
    :var gtcPackageCreationDate: The creation date of the Generic Tool Catalog package, in the format yyyymmdd_hhmmss
    """
    vendorName: str = ''
    vendorAcronym: str = ''
    gtcHierarchyVersion: str = ''
    gtcPackageId: str = ''
    vendorCatalogVersion: str = ''
    vendorCatalogLanguage: str = ''
    vendorCatalogDescription: str = ''
    vendorCatalogShortDescription: str = ''
    vendorCatalogId: str = ''
    vendorCatalogRootClassId: str = ''
    vendorCatalogRootDir: str = ''
    gtcPackageCreationDate: str = ''



from __future__ import annotations

from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class SavedQueryProperties(TcBaseObj):
    """
    Contains properties of the saved query to be created.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify returned partial errors
    associated with this input structure.
    :var queryName: Name of the saved query to be created.
    :var queryDesc: Description of the saved query to be created, may be an empty string.
    :var queryClass: The storage class name of the business object type this query will find object instances of.
    :var queryClauses: Query clauses of the saved query to be created. To make sure the query clauses is  in correct
    formate, please use Export button in Query Builder to export the saved query into a XML file, pick up the string
    value of QueryClause element in the XML, replace all " with " to get the final string, the final query clauses
    string should be the same as seen in View Properties dialog. 
    """
    clientId: str = ''
    queryName: str = ''
    queryDesc: str = ''
    queryClass: str = ''
    queryClauses: str = ''



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, RuntimeBusinessObject, Dataset
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from tcsoa.gen.Internal.StructureManagement._2013_05.StructureExpansionLite import ChildLineInfo


@dataclass
class ExpansionResponse2(TcBaseObj):
    """
    'ExpansionResponse2' contains a map with key as parent Fnd0BOMLineLite or BOMLine and values as 'ChildLineInfo'.
    Additionally it also contains 'DatasetInfo2', uids of undelivered Fnd0BOMLineLite or BOMLine objects and its
    parent. 
    
    :var parentChildInfo: Map with key as parent Fnd0BOMLineLite or BOMLine and values as 'ChildLineInfo'.
    :var datasetInfo: 'DatasetInfo2' contains name of GRM relation through which underlying object of Fnd0BOMLineLite
    or BOMLine is attached to Dataset, Dataset object and list of 'NamedRefInfo2'.
    :var parentUndeliveredChildrenUids: Parent BOMLine or Fnd0BOMLineLite and its children uids which are yet to be
    transferred to client.
    :var serviceData: The service data containing partial errors.
    """
    parentChildInfo: ParentLineToChildLineInfo2 = None
    datasetInfo: List[DatasetInfo2] = ()
    parentUndeliveredChildrenUids: ParentLineToChildrenLineUidValues = None
    serviceData: ServiceData = None


@dataclass
class NamedRefInfo2(TcBaseObj):
    """
    Contains details of named reference object of Dataset.
    
    :var name: Name of reference object in Dataset. For example, JTPART.
    :var type: The type of the named reference object.
    :var originalFileName: The 'original_file_name' attribute value of the file in case the named reference object is a
    file.
    :var object: Object reference corresponding to the named reference.
    :var fileTicket: FMS ticket used to retrieve the file in case.
    """
    name: str = ''
    type: str = ''
    originalFileName: str = ''
    object: BusinessObject = None
    fileTicket: str = ''


@dataclass
class DatasetInfo2(TcBaseObj):
    """
    'DatasetInfo2' describes GRM relation name through which Dataset is attached with underlying object of
    Fnd0BOMLineLite or BOMLine. It also contains 'NamedRefInfo2' for Dataset requested through
    'DatasetTypeAndNamedRefs'.
    
    :var relationName: Name of GRM relation through which underlying object of Fnd0BOMLineLite or BOMLine is related to
    Dataset object.
    :var dataset: Dataset attached to Fnd0BOMLineLite or BOMLine.
    :var namedRefInfo: Named reference objects details for Dataset.
    """
    relationName: str = ''
    dataset: Dataset = None
    namedRefInfo: List[NamedRefInfo2] = ()


"""
Map with key as Fnd0BOMLineLite or BOMLine and 'ChildLineInfo' as values.
"""
ParentLineToChildLineInfo2 = Dict[RuntimeBusinessObject, List[ChildLineInfo]]


"""
Map with key as Fnd0BOMLineLite or BOMLine and uids of the children lines as values.
"""
ParentLineToChildrenLineUidValues = Dict[RuntimeBusinessObject, List[str]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class LogicalObjectTypeInput(TcBaseObj):
    """
    Holds the logical object type definition data for the specified logical object type to be created.
    
    :var name: Name of the logical object type. (user supplied name)
    :var displayName: Display name of the logical object type.
    :var description: Description of the logical object type.
    :var rootTypeName: Name of the root business object, , All subtypes of POM_object are supported.
    :var parentTypeName: Name of the parent logical object type, It should be a type or subtype of Fnd0LogicalObject. 
    (optional)
    :var retrieveClassificationData: If TRUE , classification data will be retrieved for the root business object.
    If FALSE , classification data will not be retrieved for the root business object.
    :var memberPropertyDefinitions: A list of member property defintions. (optional)
    :var presentedPropertyDefinitions: A list of presented property defintions. (optional)
    """
    name: str = ''
    displayName: str = ''
    description: str = ''
    rootTypeName: str = ''
    parentTypeName: str = ''
    retrieveClassificationData: bool = False
    memberPropertyDefinitions: List[MemberPropertyDefinition] = ()
    presentedPropertyDefinitions: List[PresentedPropertyDefinition] = ()


@dataclass
class LogicalObjectTypeResponse(TcBaseObj):
    """
    Holds the list of the created logical object types and partial errors.
    
    :var loTypes: A list of the created logical object types.
    :var serviceData: Returned service data.
    """
    loTypes: List[BusinessObject] = ()
    serviceData: ServiceData = None


@dataclass
class MemberPropertyDefinition(TcBaseObj):
    """
    Member properties represents the objects, which is reached from the root business object via the specified
    traversal paths, in order to present their properties onto the logical object.
    
    :var memberPropertyName: Name of the member property.
    :var displayName: Display name of the member property.
    :var description: Description of the member property.
    :var traversalPath: A list of traversal hops.
    :var retrieveClassificationData: If TRUE , classification data will be retrieved for this member property.
    If FALSE , classification data will not be retrieved for this member property.
    """
    memberPropertyName: str = ''
    displayName: str = ''
    description: str = ''
    traversalPath: List[TraversalHop] = ()
    retrieveClassificationData: bool = False


@dataclass
class PresentedPropertyDefinition(TcBaseObj):
    """
    Presented properties represents the actual source properties of a root or member object on the logical object.
    
    :var presentedPropertyName: Name of the presented property.
    :var displayName: Display name of the presented property.
    :var description: Description of the presented property.
    :var rootOrMemberName: Root business object name or member property name.
    :var sourcePropertyName: Name of the source property of root or member business object.
    """
    presentedPropertyName: str = ''
    displayName: str = ''
    description: str = ''
    rootOrMemberName: str = ''
    sourcePropertyName: str = ''


@dataclass
class TraversalHop(TcBaseObj):
    """
    Holds traversal hop information which is represented either as a relation property (GRM, GRMS2P) or reference
    property (REF, REFBY).
    
    :var propertyName: Source object property name.
    :var propertyType: Type of property. Supported values are "relation" or "reference".
    :var destinationType: Type of destination object.
    :var direction: Direction of traversal. Supported values are "forward" or "reverse".
    """
    propertyName: str = ''
    propertyType: str = ''
    destinationType: str = ''
    direction: str = ''


@dataclass
class AddMemberAndPresentedPropsResponse(TcBaseObj):
    """
    Holds the output list of defined member and/or presented properties that were created on the specified logical
    object type and any partial errors, if thrown.
    
    :var memberOrPresentedProps: List of defined member and/or presented properties that were created on the specified
    input logical object type.
    :var serviceData: Returned service data.
    """
    memberOrPresentedProps: List[BusinessObject] = ()
    serviceData: ServiceData = None


@dataclass
class AddMembersPresentedPropsInput(TcBaseObj):
    """
    Holds the member and presented properties definition data on the specified logical object type.
    
    :var logicalObjectType: An object of logical object type ( subtype of "Fnd0LogicalObject" ) to which members and
    presented properties are to be added.
    :var memberDefinitions: A list of member defintions.
    :var presentedPropertiesDefinitions: A list of presented property defintions.
    """
    logicalObjectType: BusinessObject = None
    memberDefinitions: List[MemberPropertyDefinition] = ()
    presentedPropertiesDefinitions: List[PresentedPropertyDefinition] = ()


@dataclass
class DeleteMembersPresentedPropsInput(TcBaseObj):
    """
    Holds the logical object type and the member and presented properties on the specified logical object type.
    
    :var logicalObject: An object of type "Fnd0LogicalObject" for which members and presented properties are to be
    deleted.
    :var memberIdsOrPresentedProps: A list of member ids or presented property names.
    """
    logicalObject: BusinessObject = None
    memberIdsOrPresentedProps: List[str] = ()



from __future__ import annotations

from tcsoa.gen.Query._2019_06.SavedQuery import AttributeSortOption
from tcsoa.gen.Query._2010_09.SavedQuery import BusinessObjectQueryClause
from typing import Dict, List
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class BusinessObjectQueryInput4(TcBaseObj):
    """
    Input structure for executeBOQueriesWithSortAndContext operation.
    
    :var boTypeName: Name of business object type. Supports BusinessObject and all subtypes.
    :var clauses: Query clauses in search criteria. Each clause consists of property name on typeName, property value,
    logic and math operators.
    :var sortOptions: A list of attribute sort options based on which the results will be sorted. The first element of
    the list is the primary sort field, second is the secondary sort field and so on.
    :var maxNumToReturn: Specified maximum number of objects to return.
    :var requestId: Unique ID used to register the query execution task. This can be used by the caller to cancel the
    time consuming query, the value can be generated by any unique string generator. The query can be cancelled by
    calling: TcSoaFrameworkCore.Session.cancelOperations (requestId).
    :var clientId: This unique ID is used to identify return data elements and partial errors associated with this
    input structure. This is currently not yet used by the return data elements, the caller can leave it empty.
    :var configurationContextMap: A map (string, string) of an included logical object ID and the selected
    Configuration Context UID. An Included Logical Object ( Fnd0IncludedLODef ) is a logical object definition that has
    been added to another logical object defintion.
    """
    boTypeName: str = ''
    clauses: List[BusinessObjectQueryClause] = ()
    sortOptions: List[AttributeSortOption] = ()
    maxNumToReturn: int = 0
    requestId: str = ''
    clientId: str = ''
    configurationContextMap: NameValueMap2 = None


"""
A map of name value pairs.
"""
NameValueMap2 = Dict[str, str]



from __future__ import annotations

from tcsoa.gen.Workflow._2014_10.Workflow import CreateWkfInput, CreateWkfOutput, CreateRemoteWkfInput
from tcsoa.base import TcService


class WorkflowService(TcService):

    @classmethod
    def createRemoteWorkflowAsync(cls, createRemoteWkfInput: CreateRemoteWkfInput) -> None:
        """
        This operation submits a request to the Teamcenter Integration Framework (TcIF) to create a workflow at the
        remote site. The request is accomplished by calling the TcIF Web Service operation to invoke the workflow
        groovy process.  The groovy process orchestrates the steps involved in the creation of the workflow process.
        
        Use cases:
        Use this operation to create a workflow process at a remote site based on a specific template and with specific
        attachments and attachment types. For example: When configuring the creation of workflow task from a schedule
        task, one can assign a remote privileged user and a remote workflow owner. When the creation of workflow is
        triggered from the schedule task, the request will be submitted to the Teamcenter Integration Framework (TcIF)
        to orchestrate the replication of attachments, creation of the workflow, creation of the workflow task proxy
        link and the schedule task proxy link.
        
        Exceptions:
        >Throws a service exception if TcGS login fails or if submitting a request to create a remote workflow fails.
        """
        return cls.execute_soa_method(
            method_name='createRemoteWorkflowAsync',
            library='Workflow',
            service_date='2014_10',
            service_name='Workflow',
            params={'createRemoteWkfInput': createRemoteWkfInput},
            response_cls=None,
        )

    @classmethod
    def createWorkflow(cls, input: CreateWkfInput) -> CreateWkfOutput:
        """
        This operation creates a workflow process given the  process template name, workflow owner, responsible party,
        attachments and attachment types.
        
        Use cases:
        Any client can use this operation to create a workflow process in Teamcenter given the process template name,
        workflow owner and the responsible party.  It can also provide the attachments to attach to the workflow
        process. For Example:  Teamcenter Integration Framework can call this operation by providing the process
        template name, workflow owner, responsible party, attachments, attachment types.  Workflow process will be
        created using the given workflow template. The responsibly party and the workflow owner will be assigned to the
        root task. The attachments will be attached using the given attachment relation types. 
        
        Exceptions:
        >Throws an expectation if an unhandled error is encountered during Workflow process creation.
        """
        return cls.execute_soa_method(
            method_name='createWorkflow',
            library='Workflow',
            service_date='2014_10',
            service_name='Workflow',
            params={'input': input},
            response_cls=CreateWkfOutput,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMLine
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ExpandAndSearchOutput(TcBaseObj):
    """
    ExpandAndSearchOutput contains the result BOMLine with the satisfied search condition indexes.
    
    :var resultLine: The result BOMLine. 
    :var satisfiedConditions: The indexes of search conditions satisfied by result line. Index will start from 0.
    """
    resultLine: BOMLine = None
    satisfiedConditions: List[int] = ()


@dataclass
class ExpandAndSearchResponse(TcBaseObj):
    """
     This structure provides output for the expandAndSearch operation. ExpandAndSearchResponse contains the list of a
    structure which contains result BOMLine with the satisfied search condition indexes.
    
    :var outputLines: Structure with result BOMLine with the satisfied search condition indexes.
    :var serviceData: The service data.
    """
    outputLines: List[ExpandAndSearchOutput] = ()
    serviceData: ServiceData = None


@dataclass
class SearchCondition(TcBaseObj):
    """
    Search condition contains the information of search criteria. This will form a condion as "bl_item_item_id =
    000016" where bl_item_item_id is a property and 000016 is input value given by a user for search. User can combine
    more than one conditions with logical operator 'AND' and 'OR' like "bl_item_item_id = 000016 AND
    bl_level_starting_0 != 4 AND bl_is_precise == false" . 
    
    Allowed relational operators:
    =       Use this operator for wild card search. This operator will do a wild card search for given value,       no
    need to give a '*'.
    ==  Use this operator to search exact match of search value
    !=      Use this operator to search other values except search value
    >      Use this operator to search greater value than search value
    >=  Use this operator to search greater and equal value than search value
    <      Use this operator to search lesser value than search value
    <=  Use this operator to search lesser and equal value than search value
    
    Note: Comparison will be done on the basis of BOMLine property type only.
    
    
    :var logicalOperator: The logical operator for combining multiple search conditions. Only 'AND' and 'OR' can be
    used for combining the multiple conditions.
    :var propertyName: The property name to use for search.
    :var relationalOperator: The relational operator to use for search value comparison.
    :var inputValue: The property value to use for search.
    """
    logicalOperator: str = ''
    propertyName: str = ''
    relationalOperator: str = ''
    inputValue: str = ''



from __future__ import annotations

from tcsoa.gen.Core._2006_03.FileManagement import CommitDatasetFileInfo
from tcsoa.gen.Core._2017_05.ProjectLevelSecurity import ModifyProjectsInfo2, CopyProjectsInfo2, ProjectAssignOrRemoveInput, ProjectInformation2, ProjectsInput, PropagationDataInput, ProjectsOutputResponse
from typing import List
from tcsoa.gen.Core._2012_09.ProjectLevelSecurity import ProjectOpsResponse
from tcsoa.gen.Core._2017_05.FileManagement import ReplaceFileInput
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class ProjectLevelSecurityService(TcService):

    @classmethod
    def getProjectsForAssignOrRemove(cls, projectsInput: List[ProjectsInput]) -> ProjectsOutputResponse:
        """
        This operation retrieves the assigned projects to the input data and all available projects where the input
        user is a priveleged member. When multiple business objects are selected this operation retrieves the assigned
        projects which are in common for the complete input data. It also retrieves level or structure information in
        case of ActiveWorkspace Content context. In ActiveWorkspace Content context, if the selected object does not
        have further child objects then level or structure information will not be returned.
        """
        return cls.execute_soa_method(
            method_name='getProjectsForAssignOrRemove',
            library='Core',
            service_date='2017_05',
            service_name='ProjectLevelSecurity',
            params={'projectsInput': projectsInput},
            response_cls=ProjectsOutputResponse,
        )

    @classmethod
    def modifyProjects2(cls, modifyProjectsInfos: List[ModifyProjectsInfo2]) -> ProjectOpsResponse:
        """
        This operation modifies the given list of TC_Project objects using the input specified. The input contains new
        values for all the project properties. Values for properties other than the project team are ignored unless the
        user is the Project Administrator. 
        The entire Project Team, with the exception of the Project Administrator, is replaced with the specified team.
        Therefore, a Project Team Administrator must be specified. If the new Project Team is different than the
        current team, the user performing this operation must be either the Project Administrator or Project Team
        Administrator for the project being modified.
        """
        return cls.execute_soa_method(
            method_name='modifyProjects2',
            library='Core',
            service_date='2017_05',
            service_name='ProjectLevelSecurity',
            params={'modifyProjectsInfos': modifyProjectsInfos},
            response_cls=ProjectOpsResponse,
        )

    @classmethod
    def copyProjects2(cls, copyProjectInfos: List[CopyProjectsInfo2]) -> ProjectOpsResponse:
        """
        This operation copies the given list of TC_Project objects. The operation also copies any information which is
        in contained in the project. Data such as project team members and any objects assigned to the source project
        will also be copied to the new project. If a project with given project ID exists in the system then this
        operation will return error 101010. The operation will continue with copying the other projects.
        """
        return cls.execute_soa_method(
            method_name='copyProjects2',
            library='Core',
            service_date='2017_05',
            service_name='ProjectLevelSecurity',
            params={'copyProjectInfos': copyProjectInfos},
            response_cls=ProjectOpsResponse,
        )

    @classmethod
    def setPropagationEnabledProperties(cls, propagationDataInput: List[PropagationDataInput]) -> None:
        """
        The operation sets propagation enabled properties on a given set of objects and performs propagation. This will
        be done either in the same session or a different session depending on the dispatcher configuration. If
        dispatcher is configured and the SOA URL is set correctly in organization application then the request will be
        processed asynchronously. On the otherhand if dispatcher is not configured or the SOA URL is not correctly set
        in the organization application then the request will be processed in the same process.If the operation is
        executed in a different session and it involves a structures then an attempt will be made to recreate the BOM
        window in the new session; to achieve this operation takes a list of BOMWindow objects and re-creates the
        BOMwindow and the contents of the windows (i.e. configuration) by applying the supplied Revision Rule and
        variant configuration information. If the RevisionRuleEntryProps::unitNo is set to -1 then it considers default
        unitNo or use the input RevisionRule object with no changes. If no value specified for
        RevisionRuleEntryProps::unitNo, then the input RevisionRule object used as modified/transient rule with unitNo
        as 0. If the value of preference PSM_enable_product_configurator is true, then Product Configurator variant
        rule will be honored.
        
        Use cases:
        Set propagation enabled property in context of some configured window in structure manager or active workspace
        context editor.
        """
        return cls.execute_soa_method(
            method_name='setPropagationEnabledProperties',
            library='Core',
            service_date='2017_05',
            service_name='ProjectLevelSecurity',
            params={'propagationDataInput': propagationDataInput},
            response_cls=None,
        )

    @classmethod
    def createProjects2(cls, projectInfos: List[ProjectInformation2]) -> ProjectOpsResponse:
        """
        This operation creates TC_Project objects using the given input information. This operation also supports
        creation of TC_Project object in hierarchical configuration. If the project with given project ID exists in the
        system then this operation will return unique id violation error 101010. However, creation of rest of the
        projects will continue.
        
        Use cases:
        This operation provides the following use cases for Teamcenter objects:
        
        Use Case 1: Create parent program in project application.
        * Administartor user will open Project application. User will fill all details like ProjectID, Project Name,
        Team members. Here we are creating parent program so user will select "Use Program Security" flag.
        * Once the user input is received, client makes subsequent invocation to this operation to execute
        createProjects2.
        * The method is invoked. New program is created using values passed in. Newly created program will display on
        left side in Project application.
        
        Use Case 2: Create a project with associated program.
        * Administartor user will open Project application. User will fill all details like ProjectID, Project Name,
        Team members. Here we are creating project which will associated with
        program so user will not select "Use Program Security" flag. User will also select parent program from the list
        of values.
        * Once the user input is received, client makes subsequent invocation to this operation to execute
        createProjects2.
        * The method is invoked. New project is created using values passed in. Newly created project will display on
        left side in Project application under select parent program.
        """
        return cls.execute_soa_method(
            method_name='createProjects2',
            library='Core',
            service_date='2017_05',
            service_name='ProjectLevelSecurity',
            params={'projectInfos': projectInfos},
            response_cls=ProjectOpsResponse,
        )

    @classmethod
    def assignOrRemoveObjectsFromProjects(cls, assignOrRemoveInput: List[ProjectAssignOrRemoveInput]) -> ServiceData:
        """
        This operation assigns the given set of workspace objects to the given projects. Similarly, it removes an
        additional set of given workspace objects from the same set of given projects. If the input contains revision
        rule and or variant rule these will be applied to the given set of objects for traversing the structure i.e.
        the project will be propagated to the objects which are obtained by applying these configurations. If the
        additional input parameters type options and depth are specified; the assign or remove operation will filter
        out additional objects based on the inputs. If user is not privileged to assign objects to any of the given
        projects then this operation will return the error 101014 : You have insufficient privilege to assign object to
        a project. Similarly, if user is not privileged to remove objects from any of the given projects then this
        operation will return error 101015: You have insufficient privilege to remove object from a project. These
        errors will not terminate processing of rest of the objects.
        
        Use cases:
        - Assign projects to objects specified in an input by applying the given revision rules and variant rules for
        4GD structures. 
        - Assign project to objects specified in the input by applying the current BOM window in classic BOM 
        - Assign projects to objects specified without applying any configuration information
        
        """
        return cls.execute_soa_method(
            method_name='assignOrRemoveObjectsFromProjects',
            library='Core',
            service_date='2017_05',
            service_name='ProjectLevelSecurity',
            params={'assignOrRemoveInput': assignOrRemoveInput},
            response_cls=ServiceData,
        )


class FileManagementService(TcService):

    @classmethod
    def commitDatasetFilesInBulk(cls, commitInput: List[CommitDatasetFileInfo]) -> ServiceData:
        """
        This operation uploads files to a Teamcenter volume and associates them to a Dataset. The mechanism for a
        client application adding files to a Teamcenter volume contains several steps. This mechanism is implemented in
        the com.teamcenter.soa.client.FileManagementUtility class, which provides this functionality to clients in a
        consistent, reusable package. 
        
        This operation is supported only for internal Siemens PLM purposes. Customers should not invoke this operation.
        
        Use cases:
        This operation uploads files to a Teamcenter volume and associates them as named references of a Dataset. All
        data is bulk saved to improve performance.
        """
        return cls.execute_soa_method(
            method_name='commitDatasetFilesInBulk',
            library='Core',
            service_date='2017_05',
            service_name='FileManagement',
            params={'commitInput': commitInput},
            response_cls=ServiceData,
        )

    @classmethod
    def replaceFiles(cls, inputs: List[ReplaceFileInput]) -> ServiceData:
        """
        This operation replaces an existing volume file with a new file that has already been uploaded to a transient
        volume.  It uploads the file from the transient volume to the regular Teamcenter volume.  The original volume
        file is replaced with the new file, and the ImanFile references the new file.  Note that there is no new
        ImanFile object created.  This operation includes the ability to change the original file name or retain its
        existing value.  The file type on the ImanFile object is updated to match the value from the input write
        tickets.  The tickets for the upload to the transient volume can be obtained by calling
        'getTransientFileTicketsForUpload'.
        
        This operation is used for replacing an existing file in a volume when the original file content is to be
        replaced with a newly encrypted file.
        
        Use cases:
        There are very specific cases where this operation should be used. One use case is where encryption software
        needs to replace the contents of the original file with the encrypted contents.
        """
        return cls.execute_soa_method(
            method_name='replaceFiles',
            library='Core',
            service_date='2017_05',
            service_name='FileManagement',
            params={'inputs': inputs},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.Cad._2008_06.DataManagement import ItemInfo, ExtraObjectInfo, ItemRevInfo, AttributeInfo, NamedReferenceObjectInfo, BoundingBox
from tcsoa.gen.BusinessObjects import ImanVolume, Dataset
from typing import List
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class PartInfo(TcBaseObj):
    """
    The PartInfo struct is the main input to the createOrUpdateParts service for boundingbox. This structure refers to
    the Item, ItemRevision, and one or more Dataset structures used to create those objects.
    
    :var clientId: Identifier defined by user to track the related object.
    :var itemInput: Member of type ItemInfo
    :var itemRevInput: Member of type ItemRevInfo
    :var datasetInput: List of DatasetInfos
    """
    clientId: str = ''
    itemInput: ItemInfo = None
    itemRevInput: ItemRevInfo = None
    datasetInput: List[DatasetInfo] = ()


@dataclass
class DatasetFileInfo(TcBaseObj):
    """
    Holds the basic info for a file to be uploaded to a Dataset.
    
    :var clientId: Identifier defined by user to track the related object.
    :var fileName: Name of file to be uploaded.  Filename only, should not contain path to filename.
    :var namedReferencedName: Named Reference relation to file.
    :var isText: Flag to indicate if file is text ( TRUE ) or binary (FALSE ).
    :var allowReplace: Flag to indicate if file can be overwritten ( TRUE ) or not ( FALSE ).
    :var boundingBoxesAvailable: Flag to indicate 'BoundingBoxes' are available.
    :var destinationVolume: Destination volume into which the file will be uploaded. If null tag the current default
    volume of the user will be used.
    :var boundingBoxes: List of 'BoundingBoxes'.
    """
    clientId: str = ''
    fileName: str = ''
    namedReferencedName: str = ''
    isText: bool = False
    allowReplace: bool = False
    boundingBoxesAvailable: bool = False
    destinationVolume: ImanVolume = None
    boundingBoxes: List[BoundingBox] = ()


@dataclass
class DatasetInfo(TcBaseObj):
    """
    The DatasetInfo struct represents all of the data necessary to construct the dataset object.
    The basic attributes that are required are passed as named elements in the structure.
    All other attributes are passed as name/value pairs in the AttributeInfo structure.
    The extraObject field allows for the creation of an object(s) that will be related to this newly created Dataset.
    
    :var clientId: Identifier defined by user to track the related object.
    :var dataset: Dataset object reference for update, null for creation
    :var createNewVersion: Flag to create new version ( TRUE ) or not (FALSE )
    :var mapAttributesWithoutDataset: Flag to indicate whether DatasetInfo should be used for mapping attributes or for
    create.
    :var namedReferencePreference: Preference name which holds the list of named references to delete from one Dataset
    version to the next
    :var attrList: List of AttributeInfos for attributes
    :var mappingAttributes: List of AttributeInfos for mapped attributes. Mapped atributes are attributes that are
    applied to other objects. Refere to the ITK manual for a definition of attribute mapping.
    :var extraObject: List of ExtraObjectInfos
    :var datasetFileInfos: List of DatasetFileInfos
    :var namedReferenceObjectInfos: List of NamedReferenceObjectInfos
    :var datasetTool: The dataset tool for the dataset. If not specified then the default tool will be used.
    :var name: Name attribute value
    :var basisName: basisName
    :var description: Description attribute value
    :var type: Type attribute value
    :var lastModifiedOfDataset: lastModifiedOfDataset
    :var id: ID attribute value
    :var datasetRev: Revision attribute value
    :var itemRevRelationName: Can be null, defaulted
    """
    clientId: str = ''
    dataset: Dataset = None
    createNewVersion: bool = False
    mapAttributesWithoutDataset: bool = False
    namedReferencePreference: str = ''
    attrList: List[AttributeInfo] = ()
    mappingAttributes: List[AttributeInfo] = ()
    extraObject: List[ExtraObjectInfo] = ()
    datasetFileInfos: List[DatasetFileInfo] = ()
    namedReferenceObjectInfos: List[NamedReferenceObjectInfo] = ()
    datasetTool: str = ''
    name: str = ''
    basisName: str = ''
    description: str = ''
    type: str = ''
    lastModifiedOfDataset: datetime = None
    id: str = ''
    datasetRev: str = ''
    itemRevRelationName: str = ''



from __future__ import annotations

from tcsoa.gen.Internal.Core._2020_01.ActiveModeler import AddPropsOnTypeInput, TypeInput
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class ActiveModelerService(TcService):

    @classmethod
    def addPropertiesOnTypes(cls, addPropsOnTypeInputs: List[AddPropsOnTypeInput]) -> ServiceData:
        """
        This method adds new properties on existing Business object types. This method validates the inputs, creates
        the new properties and updates a dataset with the inputs. This dataset will be later used by BMIDE client to
        sync the custom template.
        
        Use cases:
        Add properties on type.
        """
        return cls.execute_soa_method(
            method_name='addPropertiesOnTypes',
            library='Internal-Core',
            service_date='2020_01',
            service_name='ActiveModeler',
            params={'addPropsOnTypeInputs': addPropsOnTypeInputs},
            response_cls=ServiceData,
        )

    @classmethod
    def createTypes(cls, typeInput: List[TypeInput]) -> ServiceData:
        """
        This method creates new business object types. This method will validate the inputs, create the business object
        type and create datasets with input information. The dataset will be used to sync up changes into an existing
        Business Modeler IDE (BMIDE) project.
        
        Use cases:
        Create Teamcenter Type.
        """
        return cls.execute_soa_method(
            method_name='createTypes',
            library='Internal-Core',
            service_date='2020_01',
            service_name='ActiveModeler',
            params={'typeInput': typeInput},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import User, Group, Role
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetAttributeValuesInputData(TcBaseObj):
    """
    The data structure contains class name and attribute name for 'getAttributeValue' operation.
    
    :var className: Name of class containing the attribute
    :var attributeName: Internal name of the attribute
    """
    className: str = ''
    attributeName: str = ''


@dataclass
class GetAttributeValuesOutputData(TcBaseObj):
    """
    The data structure contains class attribute names and values.
    
    :var classAttribute: Class name and attribute names
    :var values: List of string attribute values
    """
    classAttribute: GetAttributeValuesInputData = None
    values: List[str] = ()


@dataclass
class GetAttributeValuesResponse(TcBaseObj):
    """
    This data structure contains attribute values for all  instances of a class.
    
    :var output: List of class name and attribute values
    :var serviceData: serviceData
    """
    output: List[GetAttributeValuesOutputData] = ()
    serviceData: ServiceData = None


@dataclass
class GetOrganizationInformationInputData(TcBaseObj):
    """
    This structure holds the user options to retrieve the organization hierarchy.
    
    :var groupName: The name of a group. Not supported.
    :var onlyFirstLevelSubGroups: It is not supported and should not be set. All subgroups are returned.
    :var includeRoleInGroupInfo: The option to include roles in the returned group information if it is true, otherwise
    no roles of the  groups are included.
    :var includeUsersInGroupRoleInfo: The option to include users with each role belonging to a group.
    """
    groupName: str = ''
    onlyFirstLevelSubGroups: bool = False
    includeRoleInGroupInfo: bool = False
    includeUsersInGroupRoleInfo: bool = False


@dataclass
class GetOrganizationInformationResponse(TcBaseObj):
    """
    This structure holds the group hierarchical information like the parent of a group and its child groups as well as
    roles added to a group and group members if they are included.
    
    :var hierDataMap: The lookup map to find child groups for a parent group.
    :var rootLevelGroups: List of top level groups in the organization.
    :var serviceData: Object with all groups in its plain object list and possible errors in retrieving all groups,
    roles and group members.
    """
    hierDataMap: GroupHierarchyDataMap = None
    rootLevelGroups: List[GroupHierarchyData] = ()
    serviceData: ServiceData = None


@dataclass
class GroupHierarchyData(TcBaseObj):
    """
    This structure holds the group hierarchical information like the parent of a group and its child groups as well as
    roles added to a group and group members if they are included.
    
    :var group: The target group
    :var subGroups: The child groups of the target group
    :var parent: The parent group of the target group
    :var roleUsers: The  list of RoleUsers objects. Each RoleUsers object hold a role added to the target  group and
    users with the role under the target group
    """
    group: Group = None
    subGroups: List[Group] = ()
    parent: Group = None
    roleUsers: List[RoleUsers] = ()


@dataclass
class RoleUsers(TcBaseObj):
    """
    This structure contains infomariton for a role added to a group and list of users with the role under the group.
    
    :var role: The role added to a group
    :var users: The users with the role under the group
    """
    role: Role = None
    users: List[User] = ()


"""
Map of Group to 'GroupHierarchyData' '(Group, GroupHierarchyData').
"""
GroupHierarchyDataMap = Dict[Group, GroupHierarchyData]



from __future__ import annotations

from tcsoa.gen.Classification._2009_10.Classification import ClassAttribute2, KeyLOVDefinition2
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from tcsoa.gen.Classification._2007_01.Classification import ClassDef
from typing import Dict


@dataclass
class GetClassDefinitionsResponse(TcBaseObj):
    """
    Holds the values returned by getClassDefinitions operation.
    
    :var classDefinitionMap: A map of Classification class or view ID and its corresponding definition pairs
    (string/ClassDef).
    :var classAttributesDefinitionMap: A map of Classification class or view ID and its definition pairs
    (string/ClassAttributesDefinition).
    :var keyLOVDefinitionMap: A map containing the used Classification KeyLOVs and its definitions (string
    ID/KeyLOVDefinition2).
    :var serviceData: Any failures will be returned in the service data list of partial errors.
    """
    classDefinitionMap: ClassDefinitionMap = None
    classAttributesDefinitionMap: ClassAttributesDefinitionMap = None
    keyLOVDefinitionMap: KeyLOVDefinitionMap = None
    serviceData: ServiceData = None


@dataclass
class ClassAttribute3(TcBaseObj):
    """
    The structure representing class attributes.
    
    :var classAttributeProperty: A structure representing Class attribute details.
    :var dependencyAttribute: The dependency attribute property of this attribute.
    :var dependencyConfiguration: The dependency configuration property of this attribute.
    """
    classAttributeProperty: ClassAttribute2 = None
    dependencyAttribute: str = ''
    dependencyConfiguration: str = ''


@dataclass
class ClassAttributesDefinition(TcBaseObj):
    """
    The structure containing list of Classification class attributes definition and configured KeyLOV (Stxt) definition.
    
    :var attributeDefinitionMap: A map of attribute ID and attribute definition pairs (int/ClassAttribute3).
    :var configuredKeyLOVDefinitionMap: A map of attribute ID and KeyLOV definition pairs (int/KeyLOVDefinition2),
    based on dependency configuration of an attribute.
    """
    attributeDefinitionMap: AttributeDefinitionMap = None
    configuredKeyLOVDefinitionMap: ConfiguredKeyLOVDefinitionMap = None


"""
The list of Classification attributes ID and attributes property descriptor.
"""
AttributeDefinitionMap = Dict[int, ClassAttribute3]


"""
The list of KeyLOV (Stxt) id and corresponding KeyLOV (Stxt) definition.
"""
KeyLOVDefinitionMap = Dict[str, KeyLOVDefinition2]


"""
The list of Classification Class or View ID and corresponding attributes definition.
"""
ClassAttributesDefinitionMap = Dict[str, ClassAttributesDefinition]


"""
The list of Classification Class or View ID and corresponding definition.
"""
ClassDefinitionMap = Dict[str, ClassDef]


"""
The list of attribute ID and corresponding configured KeyLOV (Stxt) definition based on dependency configuration set on class attribute.
"""
ConfiguredKeyLOVDefinitionMap = Dict[int, KeyLOVDefinition2]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, WorkspaceObject
from enum import Enum
from typing import List
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class ImportExportOptions(TcBaseObj):
    """
    This structure contains additional options required to pass during the Import\ or Export operations. This structure
    is used for providing the import or export options depending on the mode of operation. This is key/value pair.  
    
    The Structure contains following element:
    
    - option         Supported option name is: CheckOutObjects.
    - optionvalue    Supported option value is: CheckOutObjects.
    
    
    
    :var option: option name
    :var optionvalue: value
    """
    option: str = ''
    optionvalue: str = ''


@dataclass
class SetContentInput2(TcBaseObj):
    """
    SetContentInput2 structure represents the input parameter required for SOA operation setRichContent2.
    
    :var objectToProcess: SpecElementRevision or FullText object
    :var bodyText: Value of the new body text on the FullText object.
    :var lastSavedDate: Last saved date of Object.
    :var contentType: The content type on the FullText object.Supported values are: Value can be REQ_HTML,
    REQ_PLAINTEXT and REQ_RICHTEXT.
    :var isPessimisticLock: This parameter is unused currently. The Behavior will be added in future releases. This is
    the placeholder for implementing optimistic/pessimistic locking provided by framework.
    """
    objectToProcess: WorkspaceObject = None
    bodyText: str = ''
    lastSavedDate: datetime = None
    contentType: ContentTypes2 = None
    isPessimisticLock: bool = False


@dataclass
class ExportInputDataAsync(TcBaseObj):
    """
    The ExportInputDataAsync represents the data required to export selected objects to MSExcel.
    
    :var objectsToExport: A list of business objects (WorkspaceObject) to be exported.
    :var targetObjectsToExport: A list of business objects (WorkspaceObject) to be exported which acts as target
    objects used for comparing the contents. This is an optional parameter and can be empty.
    :var recipeSourceObjects: The recipe of selected BOMLine objects which contains information to reconstruct them
    back and export. This is an optional parameter and can be empty.
    :var recipeTargetObjects: The recipe of target BOMLine objects which contains information to reconstruct them back
    and export. This is an optional parameter used for compare scenarios and can be empty.
    :var attributesToExport: A list of attributes to export. This is an optional parameter which can be empty.
    :var applicationFormat: The application formats supported are: 
    "MSExcel", "MExcelLive", "MSExcelReimport".
    :var templateName: The name of the MSWord or MSExcel template.
    :var exportOptions: List of options for export. Supported options are: "CheckOutObjects", "RunInBackground".
    """
    objectsToExport: List[BusinessObject] = ()
    targetObjectsToExport: List[BusinessObject] = ()
    recipeSourceObjects: List[str] = ()
    recipeTargetObjects: List[str] = ()
    attributesToExport: List[str] = ()
    applicationFormat: str = ''
    templateName: str = ''
    exportOptions: List[ImportExportOptions] = ()


@dataclass
class ExportToApplicationInputData3(TcBaseObj):
    """
    DEPRECATED: The ExportToApplicationInput represents the data required to export selected objects to MSWord or
    MSExcel.
    
    - objectsToExport          The A list of Teamcenter business objects to be exported.
    - targetObjectsToExport  The list of Teamcenter business objects to be exported which acts as target objects used
    for comparing the contents. This is aAn optional parameter and can be empty.
    - attributesToExport       The A list of attributes to export. This is an optional parameter which can be empty.
    - applicationFormat         The application formats supported are: "MSExcel ", "MSWordXMLLive", "MExcelLive",
    "MSExcelReimport", "MSWordCompare", "HTML".
    - templateIName           The name of the MSWord or MSExcel template.
    - exportOptions             List of options for export. Supported  options are:   such as keywords like 
    "CheckOutObjects".
    
    
    
    :var objectsToExport: The list of Teamcenter business objects to be exported to MSWord or MSExcel.
    :var targetObjectsToExport: The list of Teamcenter business objects to be exported which acts as target objects
    used for comparing the contents. This is an optional parameter and can be empty.
    :var attributesToExport: The list of attributes to export. This is an optional parameter which can be empty.
    :var applicationFormat: The application formats supported are :
    "MSExcel","MSWordXMLLive","MExcelLive","MSExcelReimport","MSWordCompare","HTML"
    :var templateName: The name of the MSWord or MSExcel template.
    :var exportOptions: import export option
    """
    objectsToExport: List[BusinessObject] = ()
    targetObjectsToExport: List[BusinessObject] = ()
    attributesToExport: List[str] = ()
    applicationFormat: str = ''
    templateName: str = ''
    exportOptions: List[ImportExportOptions] = ()


class ContentTypes2(Enum):
    """
    ContentTypes2 enumeration represents the valid content types of the FullText object.
    """
    REQ_HTML = 'REQ_HTML'
    REQ_PLAINTEXT = 'REQ_PLAINTEXT'
    REQ_RICHTEXT = 'REQ_RICHTEXT'



from __future__ import annotations

from tcsoa.gen.CalendarManagement._2008_06.CalendarManagement import CalendarUpdate, CalendarContainer
from tcsoa.gen.CalendarManagement._2007_06.CalendarManagement import CreateCalendarResponse
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class CalendarManagementService(TcService):

    @classmethod
    def updateCalendars(cls, tccalendarUpdates: List[CalendarUpdate]) -> ServiceData:
        """
        This operation updates the list of specified calendars and their events based on the users request to the
        application interface. This includes creating, modifying and deleting events associated with list of specified
        calendars. The information needed to update calendars is specified in the 'CalendarUpdate' structure. It
        returns service data which contains the response data from the update request. Errors will be returned in the
        list of partial errors in the 'ServiceData'.
        """
        return cls.execute_soa_method(
            method_name='updateCalendars',
            library='CalendarManagement',
            service_date='2008_06',
            service_name='CalendarManagement',
            params={'tccalendarUpdates': tccalendarUpdates},
            response_cls=ServiceData,
        )

    @classmethod
    def createCalendars(cls, newTCCalendars: List[CalendarContainer]) -> CreateCalendarResponse:
        """
        This operation creates a list of calendars based on the users request to the application interface.
         User, Resource, Schedule, Base Calendars can be created .
        The information needed to create Calendar are specified in the 'CalendarContainer' structure. It returns
        'CreateCalendarResponses'   which contains the response data from the create request .Errors will be returned
        in the list of partial errors in the 'ServiceData'.
        
        """
        return cls.execute_soa_method(
            method_name='createCalendars',
            library='CalendarManagement',
            service_date='2008_06',
            service_name='CalendarManagement',
            params={'newTCCalendars': newTCCalendars},
            response_cls=CreateCalendarResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, ImanFile
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class KeyValueArguments(TcBaseObj):
    """
    This structure represents the key/value pairs that can be attached to the Dispatcher Request.
    
    :var key: The key of the key/value pair.
    :var value: The value of the key/value pair.
    """
    key: str = ''
    value: str = ''


@dataclass
class CreateDispatcherRequestArgs(TcBaseObj):
    """
    The CreateDispatcherRequestArgs struct is used to pass in multiple sets of data to be used in a single call.  These
    structs are passed in the collection of input arguments to the function createDispatcherRequest.
    
    :var providerName: The primary objects for the request.  This usually refers to a dataset to translate but can be
    any component.
    :var serviceName: The secondary objects for the request.  This usually refers to the Item Revision containing the
    primary objects.
    :var type: The type of this request (USER SPECIFIED)
    :var primaryObjects: The provider name to process the request.
    :var secondaryObjects: The service from the above provider to process the request.
    :var priority: The priority to assign to the request.
    :var startTime: The start time to start the request.
    :var endTime: The end time at which no new requests will be created based on interval setting.  If request is still
    processing, the request WILL be completed and will not be stopped.
    :var interval: The number of times to repeat a given request.
    :var keyValueArgs: The key/value arguments for the request.
    :var dataFiles: The key/file arguments for the request.
    """
    providerName: str = ''
    serviceName: str = ''
    type: str = ''
    primaryObjects: List[BusinessObject] = ()
    secondaryObjects: List[BusinessObject] = ()
    priority: int = 0
    startTime: str = ''
    endTime: str = ''
    interval: int = 0
    keyValueArgs: List[KeyValueArguments] = ()
    dataFiles: List[DataFiles] = ()


@dataclass
class CreateDispatcherRequestResponse(TcBaseObj):
    """
    The CreateDispatcherRequestResponse struct contains the requests that were created as a result of the inputs
    specified in the CreateDispatcherRequestArgs structure above.
    
    :var requestsCreated: The Dispatcher Request objects created.
    :var svcData: The SOA Service Data.
    """
    requestsCreated: List[BusinessObject] = ()
    svcData: ServiceData = None


@dataclass
class DataFiles(TcBaseObj):
    """
    This structure represents the key/file pairs that can be attached to the Dispatcher Request.
    
    :var key: The key of the key/file pair.
    :var file: file
    """
    key: str = ''
    file: ImanFile = None



import logging
import os
from dataclasses import dataclass
from typing import List, Dict

from tcsoa.gen.Core._2006_03.FileManagement import GetDatasetWriteTicketsInputData, CommitDatasetFileInfo, \
    DatasetFileInfo, DatasetFileTicketInfo
from tcsoa.gen.Core.services import FileManagementService
from tcsoa.gen.Server import ServiceData, ErrorStack
from tcsoa.gen.BusinessObjects import BusinessObject

from tcsoa.fcc.fcc_wrapper import FccWrapper


def chunks(lst, n):
    """Yield successive n-sized chunks from lst."""
    for i in range(0, len(lst), n):
        yield lst[i:i + n]


@dataclass
class PutFileRecord:
    file_info: DatasetFileInfo
    physical_path: str
    ticket_info: DatasetFileTicketInfo = None
    volume_id: str = None


@dataclass
class GetFileResponse:
    named_ref: BusinessObject
    path: str
    partial_errors: List[ErrorStack]


@dataclass
class GetFilesEntry:
    named_ref: BusinessObject
    file_name: str


class FileManagementUtility:
    fcc: FccWrapper = None

    @classmethod
    def ensure_init(cls):
        if cls.fcc is None:
            cls.fcc = FccWrapper()

    @classmethod
    def put_files(cls, inputs: List[GetDatasetWriteTicketsInputData]) -> List[ErrorStack]:
        cls.ensure_init()
        all_partial_errors = list()
        put_file_recors = dict()
        for i in inputs:
            for dataset_info in i.datasetFileInfos:
                if dataset_info.clientId not in put_file_recors:
                    put_file_recors[dataset_info.clientId] = PutFileRecord(dataset_info, os.path.abspath(dataset_info.fileName))
                dataset_info.fileName = os.path.basename(dataset_info.fileName)
        for chunk in chunks(inputs, 100):
            get_tickets_response = FileManagementService.getDatasetWriteTickets(chunk)
            all_partial_errors.extend(get_tickets_response.serviceData.partialErrors)
            cls._process_upload_commits(get_tickets_response.commitInfo, put_file_recors)
        return all_partial_errors

    @classmethod
    def get_file_to_location(cls, named_ref: BusinessObject, destination_path: str) -> GetFileResponse:
        cls.ensure_init()
        full_path = os.path.abspath(destination_path)
        parent_dir = os.path.dirname(full_path)
        file_name = os.path.basename(full_path)

        file_read_tickets = FileManagementService.getFileReadTickets([named_ref])
        partial_errors = file_read_tickets.serviceData.partialErrors
        result = GetFileResponse(named_ref, full_path, partial_errors)
        if not file_read_tickets.tickets:
            logging.debug('No Tickets returned for the named reference')
            return result

        ticket = next(iter(file_read_tickets.tickets.values()))
        ticket_uids = cls.fcc.register_tickets([ticket])
        try:
            cls.fcc.download_file_to_location("IMD", ticket_uids[0], parent_dir, file_name)
        finally:
            cls.fcc.unregister_tickets(ticket_uids)
        return result

    @classmethod
    def get_files_to_location(cls, get_file_entries: List[GetFilesEntry], destination_dir: str) -> List[ErrorStack]:
        cls.ensure_init()
        if not os.path.isdir(destination_dir):
            raise ValueError('FMSException - ArgumentError: please provide a valid, existing destination directory.')

        named_refs = [e.named_ref for e in get_file_entries]
        file_read_tickets = FileManagementService.getFileReadTickets(named_refs)
        partial_errors = file_read_tickets.serviceData.partialErrors
        if partial_errors is None:
            partial_errors = list()
        tickets = file_read_tickets.tickets.values()
        ticket_uids = cls.fcc.register_tickets(tickets)
        try:
            file_names = [e.file_name for e in get_file_entries]
            cls.fcc.download_files_to_location("IMD", ticket_uids, destination_dir, file_names)
        finally:
            cls.fcc.unregister_tickets(ticket_uids)
        return partial_errors

    @classmethod
    def _process_upload_commits(cls, commit_infos: List[CommitDatasetFileInfo], put_file_records: Dict[str, PutFileRecord]):
        for comm_info in commit_infos:
            tickets = list()
            file_paths = list()
            for ticket_info in comm_info.datasetFileTicketInfos:
                tickets.append(ticket_info.ticket)
                put_file_info = put_file_records[ticket_info.datasetFileInfo.clientId]
                file_paths.append(put_file_info.physical_path)
            ticket_uids = cls.fcc.register_tickets(tickets)
            try:
                cls.fcc.upload_files_to_plm(ticket_uids, file_paths)
            finally:
                cls.fcc.unregister_tickets(ticket_uids)
        commit_sd = FileManagementService.commitDatasetFiles(commitInput=commit_infos)
        cls._rollback_failed_uploads(commit_sd, put_file_records)

    @classmethod
    def _rollback_failed_uploads(cls, commit_service_data: ServiceData, put_file_records: Dict[str, PutFileRecord]):
        if not commit_service_data.partialErrors:
            return

        failed_client_ids = set(partial_error.clientId for partial_error in commit_service_data.partialErrors)
        tickets = list()
        volume_ids = list()
        for failed_client_id in failed_client_ids:
            put_file_record = put_file_records[failed_client_id]
            tickets.append(put_file_record.ticket_info.ticket)
            volume_ids.append(put_file_record.volume_id)
        ticket_uids = cls.fcc.register_tickets(tickets)
        try:
            cls.fcc.rollback_files_uploaded_to_plm(ticket_uids, volume_ids)
        finally:
            cls.fcc.unregister_tickets(ticket_uids)



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, RevisionRule
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ParentChildrenInfo(TcBaseObj):
    """
    Parent and its children information.
    
    :var parentInfo: Information of parent node.
    :var chidrenInfo: Information of children nodes corresponding to parentInfo.
    """
    parentInfo: NodeInfo = None
    chidrenInfo: List[NodeInfo] = ()


@dataclass
class PropInfo(TcBaseObj):
    """
    The PropInfo structure contains the property related information that is mapped with Teamcenter property.
    
    :var propHeader: Name of the property used as column header in Excel sheet.
    :var realPropName: Real name of the property.
    :var realPropDisplayName: Real display name of the property.
    :var isRequired: True if it is a required property.
    """
    propHeader: str = ''
    realPropName: str = ''
    realPropDisplayName: str = ''
    isRequired: bool = False


@dataclass
class StructureInfoResp(TcBaseObj):
    """
    StructureInfoResp  contains the structure information for given Excel. This data structure contains column
    configuration information and cursor to be sent back to the subsequent calls.
    
    :var columnConfig: Column configuration.
    :var cursor: Cursor that represents point to which the results have been fetched. This field is used for subsequent
    calls.
    :var nodeInfos: Pagefull of the nodes that have been fetched.
    :var serviceData: Contains any errors that might have occurred as part of the service invocation.
    """
    columnConfig: ColumnConfig = None
    cursor: Cursor = None
    nodeInfos: List[NodeInfo] = ()
    serviceData: ServiceData = None


@dataclass
class ColumnConfig(TcBaseObj):
    """
    The column configuration data.
    
    :var columnConfigId: Column configuration ID.
    :var operationType: Operation type used for determining columns.  Supported values are: "Union" and "Intersection".
    :var columns: A list of available columns.
    """
    columnConfigId: str = ''
    operationType: str = ''
    columns: List[ColumnDefInfo] = ()


@dataclass
class ColumnDefInfo(TcBaseObj):
    """
    Column Definition
    
    :var displayName: The display name for the value displayed in the column header.
    :var propertyName: The property name for the value displayed in the column.
    :var pixelWidth: The pixel width for the column. Valid pixel widths are integer values between 1 and 500.
    """
    displayName: str = ''
    propertyName: str = ''
    pixelWidth: int = 0


@dataclass
class Cursor(TcBaseObj):
    """
    Cursor is a cursor that is returned for use in subsequent call to retrieve the next page of nodes.
    
    :var startReached: If true, the first page of the results has been reached.
    :var endReached: If true, the last page of the results has been reached.
    :var startIndex: Indicates the Cursor start position for the result nodes returned so far.
    :var endIndex: Indicates the Cursor end position for the result nodes returned so far.
    :var pageSize: Indicates the maximum number of nodes that can be returned in one service call.
    """
    startReached: bool = False
    endReached: bool = False
    startIndex: int = 0
    endIndex: int = 0
    pageSize: int = 0


@dataclass
class ExcelImportInput(TcBaseObj):
    """
    ExcelImportInput provides file ticket and mapped property info to get back the structure information. It also
    contains cursor that represents point to which the results have to be fetched.
    
    :var transientFileWriteTicket: The write ticket of the Excel file to be imported to Teamcenter.
    :var propInfos: A list of property related information from the input Excel sheet that are mapped with Teamcenter
    Item, BOMLine properties.
    :var typePropInfos: A map (string, list of PropInfo) of object type to property related information to be retrieved
    for the types found in input Excel sheet.
    :var cursor: Cursor that represents point to which the results have been returned.
    """
    transientFileWriteTicket: str = ''
    propInfos: List[PropInfo] = ()
    typePropInfos: TypeMapInfo = None
    cursor: Cursor = None


@dataclass
class GroupName(TcBaseObj):
    """
    The GroupName structure contains the real and display name of the group.
    
    :var realName: Real name of the group.
    :var dispName: Display name of the group.
    :var isModifiable: True if the mapping group can be modified by logged in user.
    """
    realName: str = ''
    dispName: str = ''
    isModifiable: bool = False


@dataclass
class ImportExcelData(TcBaseObj):
    """
    The ImportExcelData structure contains all the data required to import a file to Teamcenter.
    
    :var selectedObject: Selected Object under which the product will be created. The supported types are: Folder,
    ItemRevision, and Awb0Element.
    :var transientFileWriteTicket: Write ticket of the Excel file to be imported toTeamcenter.
    :var mappingGroupData: A list of mapping groups to be consumed during the import.
    :var importOptions: A list of options to be used during the import. 
    Supported options are: 
    "ParseHeader" -  The header row in the Excel sheet is    parsed. 
    "RunInBackground" -  The excel file import runs in background mode and user will get a notification when complete.
    If omitted, after completion of import, the top line Revision of the created structure will be opened.
    :var propInfos: List of object types and property related information to be     retrieved for the types found in
    input Excel sheet. The list is used only when importOption is set to "ParseHeader".
    :var typePropInfos: A map (string, list of PropInfo) of object type to property related information to be retrieved
    for the types found in input Excel sheet.
    :var actionInfo: A map (string, string) of Excel row index to action. The map will be used to indicate an operation
    on each object. Operation is defined as the value of the map. Keys and values are case sensitive. Key contains the 
    Excel row index . Allowed values are: "Reference", "Overwrite", "Revise".     
    "Reference" - No updates to the existing Teamcenter data.                                      
    "Overwrite" - Updates the existing Teamcenter data with the input file.                               "Revise" -
    Existing data will be revised to latest revision in Teamcenter.
    """
    selectedObject: BusinessObject = None
    transientFileWriteTicket: str = ''
    mappingGroupData: MappingGroupData = None
    importOptions: List[str] = ()
    propInfos: List[PropInfo] = ()
    typePropInfos: TypeMapInfo = None
    actionInfo: ActionInfo = None


@dataclass
class ImportExcelResp(TcBaseObj):
    """
    The ImportExcelResp structure contains the list of objects of type ItemRevision.
    
    :var revObjects: A list of business objects of type ItemRevision.
    :var revisionRule: Working, Any Status RevisionRule which will be used to open imported object with Working, Any
    Status.
    :var requestPreference: The map which can have a key and value pair, used for import options. The key and value are
    case sensitive.
    :var serviceData: Service data.
    """
    revObjects: List[BusinessObject] = ()
    revisionRule: RevisionRule = None
    requestPreference: RequestPreference = None
    serviceData: ServiceData = None


@dataclass
class NodeInfo(TcBaseObj):
    """
    NodeInfo is the information about node.
    
    :var nodeId: Unique identifier (UID) of a node.
    :var displayName: Display name of a node.
    :var underlyingObjectType: Internal name of the underlying object a node presents.
    :var numberOfChildren: Number of children of the node.
    :var propNameValueMap: A map of property internal name as key and display value as map value.
    """
    nodeId: str = ''
    displayName: str = ''
    underlyingObjectType: str = ''
    numberOfChildren: int = 0
    propNameValueMap: PropNameValueMap = None


@dataclass
class MappingGroupData(TcBaseObj):
    """
    The MappingGroupData structure contains the real and display name of the mapping group along with the mapping
    information.
    
    :var groupName: It contains the real and display name of the group.
    :var actionName: Supported action: "CREATE", "UPDATE" or "DELETE".
    "CREATE" &ndash; A new Object will be created.
    "UPDATE" &ndash; Existing object will be updated.
    "DELETE" &ndash; Existing object will be deleted.
    :var mappingInfo: List of property related information.
    """
    groupName: GroupName = None
    actionName: str = ''
    mappingInfo: List[PropInfo] = ()


"""
A map (string, string) of Excel row index to action. The map will be used to indicate an operation on each object. Operation is defined as the value of the map. Keys and values are case sensitive. Key contains the  Excel row index . Allowed values are: "Reference", "Overwrite", "Revise".     
"Reference" - No updates to the existing Teamcenter data.                                      
"Overwrite" - Updates the existing Teamcenter data with the input file.                              
"Revise" - Existing data will be revised to latest revision in Teamcenter.
"""
ActionInfo = Dict[str, str]


"""
A map (string, list of PropInfo) of object type to property related information to be retrieved for the types found in input Excel sheet.
"""
TypeMapInfo = Dict[str, List[PropInfo]]


"""
PropNameValueMap is a map containing property display value for each property internal name.
"""
PropNameValueMap = Dict[str, str]


"""
The map which can have a key and value pair, used for import options. The key and value are case sensitive.
"""
RequestPreference = Dict[str, List[str]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, ImanRelation, WorkspaceObject, Dataset
from tcsoa.gen.ImportExport._2008_06.FileImportExport import TemplateOverride, ConfiguredTemplate
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ExportToApplicationInputData2(TcBaseObj):
    """
    The ExportToApplicationInputData2 structure represents all of the data necessary to export selected objects to
    specific application like MSWord and MSExcel.
    
    :var objectsToExport: lThe list of Teamcenter business objects to export.
    :var attributesToExport: The list of attributes to export.
    :var applicationFormat: The application format such as "'MSWordXML'", "'MSWordXMLLive'","'MSExcel'" and
    "'MSExcelLive'","'MSExcelReimport'","'StructureOnly'","'StructureWithContents'","'MSWordXMLLiveMarkupFN'",
    "'MSWordXMLFN'","'MSWordXMLLiveFN'"
    
    Supported application format for this operation
    
    - MSWordXML     This format is used for exporting Workspace objects to static MSWord application.
    - MSExcel    This format is used for exporting Teamcenter objects to static MSExcel  application.
    - CSV    This format is used for exporting Teamcenter objects in a comma separated file format used for audit
    purpose.
    - MSExcelLive    This format is used for exporting Teamcenter objects to Live MSExcel  application.
    - MSExcelReimport    This format is used for exporting Teamcenter objects to MSExcel  application for reimport
    purpose.
    - MSExcelLiveBulkMode    This format is used for exporting Teamcenter objects to MSExcel  application for Bulk Live
    mode so that user can make all the property changes and save all the changes to Teamcenter on click of "Save to
    Teamcenter" button.
    - MSWordXMLLive    This format is used for exporting objects of type WorkspaceObject to Live  MSWord application.
    - StructureOnly    This format is used for exporting Workspace objects to MSWord without its  contents.(only
    object_name property value exported to MSWord)
    - StructureWithContents    This format is used for exporting WorkspaceObject to MSWord Live and so that user can
    modify its contents and save them back to Teamcenter.
    - MSWordXMLLiveMarkupFN    This format is used for exporting Markups to MSWord Live using FindNo as sorting key.
    - MSWordXMLLiveMarkup    This format is used for exporting Markups to MSWord Live.
    - MSWordXMLFN    This format is used for exporting WorkspaceObject to static MSWord using FindNo as sorting key.
    - MSWordXMLLiveFN    This format is used for exporting WorkspaceObject to Live MSWord application using FindNo as
    sorting key.
    
    
    :var templateId: The name of the MSWord/MSExcel template
    :var templateType: Type of export template.
    :var objectTemplateOverride: The runtime object template override which is association of objectTemplate for a
    Business object type for a chosen SpecTemplate.
    :var exportOptions: The options to be considered during the export process like "CheckOutObjects"
    """
    objectsToExport: List[BusinessObject] = ()
    attributesToExport: List[str] = ()
    applicationFormat: str = ''
    templateId: str = ''
    templateType: str = ''
    objectTemplateOverride: List[TemplateOverride] = ()
    exportOptions: List[ImportExportOptions] = ()


@dataclass
class ExportToApplicationResponse1(TcBaseObj):
    """
    ExportToApplicationResponse structure represents the output of export to application operation.  It has information
    about file ticket for the exported file generated at the server and the single markup file sent to the client.
    
    :var transientFileReadTickets: The transient file read tickets for the exported file.
    :var markupXMLReadTickets: The transient file read tickets for the generated markup file during Export to Word and
    when the application format is "MSWordXMLLiveMarkup" or "MSWordXMLLiveMarkupFN"
    
    :var serviceData: The Service Data
    """
    transientFileReadTickets: List[str] = ()
    markupXMLReadTickets: List[str] = ()
    serviceData: ServiceData = None


@dataclass
class FileMetaData(TcBaseObj):
    """
    This structure contains the information about file path, The key will contain "FilePath" and data will contain the
    physical file path of the file to be used for the purpose of import/export.
    
    
    :var key: Placeholder for a string called "FilePath".
    :var data: Physical file path of the file for import/export to Teamcenter.
    """
    key: str = ''
    data: str = ''


@dataclass
class GetExportTemplateResponse1(TcBaseObj):
    """
    This structure contains the information about the export templates that are queried from database. If there is any
    error during the querying of export templates from the database or if there is any error during the operation, then
    it is added as part of ServiceData.
    
    :var serviceData: The Service Data
    :var outputTmplNames: The object type of template as key and a vector of template names as the value.
    
    :var configuredSpecTemplates: This parameter is not being used.
    :var configuredObjectTemplates: This parameter is not being used.
    """
    serviceData: ServiceData = None
    outputTmplNames: TemplateTypeVsTemplates = None
    configuredSpecTemplates: List[ConfiguredTemplate] = ()
    configuredObjectTemplates: List[ConfiguredTemplate] = ()


@dataclass
class GetTemplateInput(TcBaseObj):
    """
    This structure contains information about the templates to be queried from Teamcenter database. It will contain
    additional information in the form of name-value pair to store the information about the application domain.
    
    :var templateType: The type of template like "SpecTemplate", "ObjectTemplate" and "ExcelTemplate"
    
    :var nameValueMap: The name value pair to store information about the application domain.
    """
    templateType: str = ''
    nameValueMap: NameValueMap = None


@dataclass
class ImportExportOptions(TcBaseObj):
    """
    This structure contains additional options required to pass during the Import\Export operations. importOptions:
    This structure is used for providing the import or export options depending on the mode of operation. This is
    key/value pair.  
    Following are the available options used during importing data to Teamcenter
    - UnCheckOutObjects - Performs uncheckout operation
    - SyncTeamcenter - Synchronize Teamcenter data with document
    - FeedbackRequired - Runs SOA in "feedback" mode, so check conflicting objects
    - SyncDocument - Synchronize document with Teamcenter
    - ReviseAllObjects - Revise option (not used)
    - ReviseOverwriteObjects - Revise option for "modified" objects after export
    - IgnoreCheckoutObjects - Ignore checked out objects and save remaining objects.
    - CheckInObjects - Check in objects
    - UnCheckOutObjects - UnCheckout objects
    
    
    Following are the available options used during exporting data to Teamcenter
    - ShowTransferModeWarning - to show warning if secondary objects are exported to Word Live with structure editing.
    - CheckOutObjects - Checkout objects.
    
    
    
    :var option: The option name can be as below
    - UnCheckOutObjects
    - SyncTeamcenter
    - FeedbackRequired
    - SyncDocument
    - ReviseAllObjects
    - ReviseOverwriteObjects
    - IgnoreCheckoutObjects
    - CheckInObjects
    
    
    :var value: The option's value can be as below
    - UnCheckOutObjects
    - SyncTeamcenter
    - FeedbackRequired
    - SyncDocument
    - ReviseAllObjects
    - ReviseOverwriteObjects
    - IgnoreCheckoutObjects
    - CheckInObjects
    
    """
    option: str = ''
    value: str = ''


@dataclass
class ImportFromApplicationInputData2(TcBaseObj):
    """
    The ImportFromApplicationInputData2 structure represents all of the data necessary to import a specification into
    Teamcenter or import templates to Teamcenter.
    
    :var transientFileWriteTicket: The file ticket of the. docx file to be imported into Teamcenter.
    :var applicationFormat: The application format such as "'MSWordXML'","MSWordXMLLive", "MSWordXMLOverWriteCheck",
    "MSWordSpecTemplate", "MSWordObjTemplate", "MSExcelTemplate" and "MSWordSetContent"
    
    Supported application formats for this operation
    
    - MSWordXML     This format is used for importing a MSWord document to Teamcenter.
    - MSWordXMLLive    This format is used for importing a Live MSWord document to 
    
    
                                        Teamcenter.
    - MSWordXMLExisting    This format is used for importing a  MSWord document and 
    
    
                                                create a Specification within an existing chosen Specification.
    - MSWordSetContent    This format is used for importing a  Live MSWord document to  
    
    
                                            Teamcenter. This format is used by the embedded viewer to  
                                            set the rich text of SpecElement and to set the properties on the  
                                            SpecElement.
    - MSWordSpecTemplate    This format is used for importing a Specification template to  
    
    
                                                Teamcenter.
    - MSWordObjectTemplate    This format is used for importing a ObjectTemplate to 
    
    
                                                    Teamcenter.
    - MSWordExcelTemplate    This format is used for importing a ExcelTemplate to 
    
    
                                                    Teamcenter.
    :var createSpecElementType: The subtype of SpecElement to be created.
    :var specificationType: The subtype of Specification to be created.
    :var isLive: Flag to determine static or live option. Default value is non live.
    :var selectedBomLine: The top BOMLine under which new structure gets imported.
    :var fileMetaDatalist: The list of structures that contains information about the physical file path of the file.
    :var importOptions: The additional options that user can pass for import purpose.
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with the input structure.
    """
    transientFileWriteTicket: str = ''
    applicationFormat: str = ''
    createSpecElementType: str = ''
    specificationType: str = ''
    isLive: bool = False
    selectedBomLine: BusinessObject = None
    fileMetaDatalist: List[FileMetaData] = ()
    importOptions: List[ImportExportOptions] = ()
    clientId: str = ''


@dataclass
class ImportFromApplicationOutputData(TcBaseObj):
    """
    The structure contains the information about the data that is passed back to the client after the result of
    importFromApplication SOA operation. It contains the UID of the BOMWindow after the document is imported to
    Teamcenter.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this input structure.
    
    :var transientFileTickets: The file ticket of the transient file to be imported.
    :var resultObjects: The objects like the BOMWindow tag if the import is successful.
    """
    clientId: str = ''
    transientFileTickets: List[str] = ()
    resultObjects: List[BusinessObject] = ()


@dataclass
class ImportFromApplicationResponse1(TcBaseObj):
    """
    ImportFromApplicationResponse1 structure represents the output of import from application operation. It contains
    UID of the BOMWindow created after the document is imported.  In case of import of templates, it contains the
    business object of the tenplate after the import operation.
    
    :var serviceData: The Service Data. It holds the error stack which contains information about any errors that are
    generated during importFromApplication operation.
    
    :var output: The list of structure containing the resultant objects which contains the UID of the BOMWindow created
    after the document is imported.  In case of importing templates, it contains the business object of the template.
    It also contains the file ticket of the xml file containing the details about the generated errors at the server
    during importFromApplication operation.
    """
    serviceData: ServiceData = None
    output: List[ImportFromApplicationOutputData] = ()


@dataclass
class MarkupReqInput(TcBaseObj):
    """
    Input structure containing the Single Markup XML info and Markup dataset info to be created
    
    :var clientId: This is not used.
    :var markupDataset: This is not used.
    :var reqObj: This is not used.
    :var isLiveMarkup: This should be always false as we do not support markup in live mode.
    
    :var transientFileWriteTickets: FMS ticket of "mrk" file.
    """
    clientId: str = ''
    markupDataset: DatasetInfo = None
    reqObj: WorkspaceObject = None
    isLiveMarkup: bool = False
    transientFileWriteTickets: List[str] = ()


@dataclass
class MarkupReqOutput(TcBaseObj):
    """
    This structure containing the information about the created markup Dataset and the GRM relation on the created
    markup Dataset to the FullText.
    
    :var markupDatset: The created markup Dataset.
    :var relation: The GRM relation on the created Dataset to the FullText.
    """
    markupDatset: Dataset = None
    relation: ImanRelation = None


@dataclass
class MarkupReqResponse(TcBaseObj):
    """
    Structure containing markup output with created markup Dataset information and ServiceData.
    
    :var markupInfo: This is not used.
    :var serviceData: Updated markup Dataset are added to updated object list of ServiceData.
    """
    markupInfo: MarkupReqOutputMap = None
    serviceData: ServiceData = None


@dataclass
class MarkupReqUpdateInput(TcBaseObj):
    """
    Markup dataset sent by the client that needs to be updated
    
    :var markupDataset: Markup dataset sent by the client that needs to be updated
    :var reqObj: Requirement object ID which has the markups that need an update
    :var isLiveMarkup: Flag indicating whether markup is on live or non live document
    :var transientFileWriteTickets: Contains an FMS ticket of the single XML file uploaded
    """
    markupDataset: Dataset = None
    reqObj: WorkspaceObject = None
    isLiveMarkup: bool = False
    transientFileWriteTickets: List[str] = ()


@dataclass
class MarkupReqUpdateResponse(TcBaseObj):
    """
    Structure containing markup dataset update response
    
    :var serviceData: Updated markup dataset will be returned in the ServiceData. Any failure will be returned with
    error message in the ServiceData list of partial errors.
    """
    serviceData: ServiceData = None


@dataclass
class DatasetInfo(TcBaseObj):
    """
    This structure contains the markup dataset information like name, type, tool information specified by client.
    
    :var tool: The tool used to open the created Dataset.
    :var datasetType: The type of the Dataset to be created.
    :var datasetName: The name of the markup Dataset to be created.
    :var datasetDesc: The description of the markup Dataset to be created.
    """
    tool: str = ''
    datasetType: str = ''
    datasetName: str = ''
    datasetDesc: str = ''


"""
This map contains the key as clientId to the 'MarkupReqOutput' structure. The 'MarkupReqOutput' structure contains the created Markup Datatset and the GRM relation on the created markup Dataset and the FullText.
"""
MarkupReqOutputMap = Dict[str, MarkupReqOutput]


"""
This map will hold the name value pair of input. The map stores the "ApplicationDomain" as key and value of the key specific to the domain.
"""
NameValueMap = Dict[str, str]


"""
This map contains information about the template type and the name of the templates. Template type can be SpecTemplate and ObjectTemplate. For each template type the name of the templates in the database is queried and returned to the user.
"""
TemplateTypeVsTemplates = Dict[str, List[str]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, WorkspaceObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ObjectInfoLegacy(TcBaseObj):
    """
    This structure holds object information of source or target object of trace link in Report structure.
    
    :var contextName: This parameter will hold the name of context for source or target object of
     trace link.
    :var displayObj: If the trace link is created on the occurrence then this variable will represent the object to
    show in the trace report.
    
    
    :var object: This object represent any Business Object in Teamcenter on which Trace Link is 
    created.
    
    :var bomView: This field represents the PSBOMView of context line for the trace link.
    :var contextLineObj: This field represents the context line for the trace link, this will be PSBOMViewRevision
    object.
    """
    contextName: str = ''
    displayObj: WorkspaceObject = None
    object: BusinessObject = None
    bomView: BusinessObject = None
    contextLineObj: BusinessObject = None


@dataclass
class ReportLegacy(TcBaseObj):
    """
    This structure holds information about each trace link report with information of parent object, and its children
    associated with trace link relation.
    
    :var parent: Parent object of trace link.
    :var children: List of all child objects for the parent with trace link relation.
    """
    parent: ObjectInfoLegacy = None
    children: List[ObjectInfoLegacy] = ()


@dataclass
class TraceReport2(TcBaseObj):
    """
    This structure holds information about generated trace reports with defining and complying trace trees with
    selected objects, and its persistent objects.
    
    :var defRootNode: The root node of defining tree in the trace report.
    :var compRootNode: The root node of complying tree in the trace report.
    :var selectedObject: The selected object for which trace report generated.
    :var persistentObjects: Persistent objects for the selected object for which Trace Report generated.
    """
    defRootNode: TraceReportTreeNode2 = None
    compRootNode: TraceReportTreeNode2 = None
    selectedObject: BusinessObject = None
    persistentObjects: List[BusinessObject] = ()


@dataclass
class TraceReportLegacy(TcBaseObj):
    """
    This structure holds information about generated trace reports with defining and complying trace trees with
    selected objects, and its persistent objects.
    
    :var definingTree: List of objects in the Defining Tree in the Trace Report giving all defining trace link details.
    
    :var indirectDefiningTree: List of objects in Indirect Defining Tree in the Trace  Report.
    :var complyingTree: List of objects in the Complying Tree in the Trace Report giving all complying trace link
    details.
    
    :var indirectComplyingTree: List of objects in Indirect Complying Tree in the Trace  Report.
    :var selectedObject: This structure member represents the object on which Trace Report is generated.
    :var persistentObjects: List of persistent object for the selected object.
    """
    definingTree: List[ReportLegacy] = ()
    indirectDefiningTree: List[ReportLegacy] = ()
    complyingTree: List[ReportLegacy] = ()
    indirectComplyingTree: List[ReportLegacy] = ()
    selectedObject: BusinessObject = None
    persistentObjects: List[BusinessObject] = ()


@dataclass
class TraceReportTreeNode2(TcBaseObj):
    """
    This structure holds information of each trace link report line with information of parent object, and its children
    associated with trace link relation.
    
    :var object: BusinessObject instance in Teamcenter on which Trace Link is created.
    :var displayObj: If the trace link is created on the occurrence then this variable will represent   the object to
    show in the trace report.
    :var srcContextName: Context name for source object of trace link.
    :var tarContextName: Context name for target object of trace link.
    :var bomView: The PSBOMView of context line for the Trace Link.
    :var contextLineObj: The context line (PSBOMViewRevision) for the Trace Link
    :var isDirectLink: If true, the object is a direct trace line object.
    :var isTraceLinkObj: If true, the current object  is a trace  link.
    :var childNodes: Child nodes with trace link relation with current object.
    """
    object: BusinessObject = None
    displayObj: WorkspaceObject = None
    srcContextName: str = ''
    tarContextName: str = ''
    bomView: BusinessObject = None
    contextLineObj: BusinessObject = None
    isDirectLink: bool = False
    isTraceLinkObj: bool = False
    childNodes: List[TraceReportTreeNode2] = ()


@dataclass
class TraceabilityReportOutput2(TcBaseObj):
    """
    TraceabilityReportOutput2 structure holds information of either of following including service data:
    1.    Generated trace reports with defining and/or complying trace trees with selected objects, and its persistent
    objects
    2.    Information about the file ticket for exported file generated at the server.
    
    The following partial error may be returned:
    1.     223201: MS Excel's outlining feature is limited to 8 levels, and the selected structure exceeds that level.
    Therefore it will be exported as a flat list with the "Level" column added.
    2.    223209: The Trace View template <template_name> is invalid. Please select an appropriate template as per the
    report type of trace view.
    3.    223036: Some of the objects were skipped during the export operation as no matching rule was found in the
    <sheet_name> sheet of the Excel template <template_name>.
    4.    214411: No input objects found. Please select the object to get traceability report.
    
    
    
    :var traceReports: All of the requested Trace Reports.
    :var transientFileReadTickets: The transient file read tickets for the exported file, this is specific to Export to
    Excel.
    :var serviceData: Service Data.
    """
    traceReports: List[TraceReport2] = ()
    transientFileReadTickets: List[str] = ()
    serviceData: ServiceData = None


@dataclass
class TraceabilityReportOutputLegacy(TcBaseObj):
    """
    TraceabilityReportOutputLegacy structure holds information about generated trace reports with defining and
    complying trace trees with selected objects and its persistent objects, including service data.
    
    The following partial errors may be returned:
    214311: No input objects found. Please select the object to get traceability report. 
    214312: The provided report type value is not correct; allowed values are 1,2 and 3.
    214313: The provided report depth value is not correct; please provide any value greater than zero.
    
    
    :var traceReports: This member holds all the Trace Reports user has asked for. This is list of 
    TraceReport type of structures.
    :var serviceData: Service Data.
    """
    traceReports: List[TraceReportLegacy] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Internal.AWS2._2016_03.Finder import SearchFilterField, ColumnConfig, ObjectsGroupedByProperty
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class SearchFilter2(TcBaseObj):
    """
    A structure representing a string, date or numeric type of search filter.
    
    :var searchFilterType: The type of search filter. Supported values are "StringFilter", "DateFilter",
    "NumericFilter".
    :var stringValue: The value for a string filter. This field is applicable only if the "searchFilterType" field is
    set to "StringFilter".
    :var colorValue: The color of the Search Filter.
    :var stringDisplayValue: The display value for a string filter. This field is applicable only if the
    "searchFilterType" field is set to "StringFilter".
    :var startDateValue: The starting value for a date filter. This field is applicable only if the "searchFilterType"
    field is set to "DateFilter"
    :var endDateValue: The ending value for a date filter. This field is applicable only if the "searchFilterType"
    field is set to "DateFilter"
    :var startNumericValue: The starting value for a numeric filter. This field is applicable only if the
    "searchFilterType" field is set to "NumericFilter".
    :var endNumericValue: The ending value for a numeric filter. This field is applicable only if the searchFilterType
    is set to "NumericFilter".
    :var count: The number of values in the filter. This field is populated on the service response and is ignored on
    the service input.
    :var selected: A flag that indicates if the filter was previously selected and used to filter the results.
    :var startEndRange: The interval used to generate a range facets. E.g for Date range facets, +1DAY, +1WEEK, +1MONTH
    , +1YEAR are acceptable values.  Note: Currently this value is not being used. To be supported in the future.
    """
    searchFilterType: str = ''
    stringValue: str = ''
    colorValue: str = ''
    stringDisplayValue: str = ''
    startDateValue: datetime = None
    endDateValue: datetime = None
    startNumericValue: float = 0.0
    endNumericValue: float = 0.0
    count: int = 0
    selected: bool = False
    startEndRange: str = ''


@dataclass
class SearchResponse2(TcBaseObj):
    """
    A service response structure containing search results and column configurations.
    
    :var searchResults: List of business objects obtained after performing a search.
    :var totalFound: Total number of business objects found.
    :var totalLoaded: Total number of business objects loaded.
    :var searchFilterMap2: A map (string, list of SearchFilter2) containing the list of search filters for each search
    filter field based on the search results.
    :var searchFilterCategories: A list of search filter categories ordered by filter priority.
    :var defaultFilterFieldDisplayCount: The default number of search filter categories to display.
    :var objectsGroupedByProperty: The structure containing an internal property name and a map of objects to the
    property group id. It also contains a list of unmatched objects which do not match any group.
    :var columnConfig: Effective column configuration for the client scope URI.
    :var serviceData: The service data object.
    :var endIndex: Cursor end position for the results returned so far. This value, 'endIndex', correlates with
    'startIndex' in SearchInput.
    """
    searchResults: List[BusinessObject] = ()
    totalFound: int = 0
    totalLoaded: int = 0
    searchFilterMap2: SearchFilterMap2 = None
    searchFilterCategories: List[SearchFilterField] = ()
    defaultFilterFieldDisplayCount: int = 0
    objectsGroupedByProperty: ObjectsGroupedByProperty = None
    columnConfig: ColumnConfig = None
    serviceData: ServiceData = None
    endIndex: int = 0


"""
A map containing a list of search filters for each search filter field.
"""
SearchFilterMap2 = Dict[str, List[SearchFilter2]]
